@@ -495,8 +495,8 @@ blecent_on_sync(void)
  *
  * @return int NOTE: this function should never return!
  */
-static int
-main_fn(int argc, char **argv)
+int
+mynewt_main(int argc, char **argv)
 {
     int rc;
 
@@ -523,16 +523,3 @@ main_fn(int argc, char **argv)
 
     return 0;
 }
-
-int
-main(int argc, char **argv)
-{
-#if BABBLESIM
-    extern void bsim_init(int argc, char** argv, void *main_fn);
-    bsim_init(argc, argv, main_fn);
-#else
-    main_fn(argc, argv);
-#endif
-
-    return 0;
-}
