@@ -229,7 +229,7 @@ blehr_on_sync(void)
  * @return int NOTE: this function should never return!
  */
 int
-main(void)
+mynewt_main(int argc, char **argv)
 {
     int rc;
 
