@@ -28,9 +28,6 @@
 
 #include "host/ble_gatt.h"
 #include "host/ble_gap.h"
-#if (MYNEWT_VAL(BLE_ISO_BROADCAST_SOURCE))
-#include "audio/ble_audio_broadcast_source.h"
-#endif
 
 #ifdef __cplusplus
 extern "C" {
@@ -222,7 +219,11 @@ int btshell_broadcast_start(uint8_t adv_instance);
 int btshell_broadcast_stop(uint8_t adv_instance);
 #endif
 
+#if MYNEWT_VAL(BLE_AUDIO)
 void btshell_leaudio_init(void);
+#else
+#define btshell_leaudio_init()
+#endif /* BLE_AUDIO */
 
 int btshell_gap_event(struct ble_gap_event *event, void *arg);
 void btshell_sync_stats(uint16_t handle);
