@@ -26,6 +26,7 @@
 
 #define STR_NULL        "null"
 
+#if (MYNEWT_VAL(BLE_AUDIO))
 #if (MYNEWT_VAL(BLE_ISO_BROADCAST_SOURCE))
 #include "audio/ble_audio_broadcast_source.h"
 int
@@ -391,17 +392,14 @@ cmd_leaudio_broadcast_stop(int argc, char **argv)
 #if (MYNEWT_VAL(BLE_AUDIO_BROADCAST_SINK))
 #include "audio/ble_audio_broadcast_sink.h"
 
-#define BROADCAST_SINK_PA_SYNC_TIMEOUT_DEFAULT  0x07D0
+#define BROADCAST_SINK_PA_SYNC_SKIP_DFLT    0x0000
+#define BROADCAST_SINK_PA_SYNC_TIMEOUT_DFLT 0x07D0
 
-static int
-broadcast_sink_pa_sync_params_get(struct ble_gap_periodic_sync_params *params)
-{
-    params->skip = 0;
-    params->sync_timeout = BROADCAST_SINK_PA_SYNC_TIMEOUT_DEFAULT;
-    params->reports_disabled = false;
-
-    return 0;
-}
+static struct ble_gap_periodic_sync_params broadcast_sink_periodic_sync_params = {
+    .skip = BROADCAST_SINK_PA_SYNC_SKIP_DFLT,
+    .sync_timeout = BROADCAST_SINK_PA_SYNC_TIMEOUT_DFLT,
+    .reports_disabled = false
+};
 
 static void
 codec_specific_config_printf(const struct ble_audio_codec_id *unused, const uint8_t *data,
@@ -478,10 +476,11 @@ broadcast_sink_action_fn(struct ble_audio_broadcast_sink_action *action, void *a
     switch (action->type) {
     case BLE_AUDIO_BROADCAST_SINK_ACTION_PA_SYNC:
         console_printf("PA Sync:\n");
-        return broadcast_sink_pa_sync_params_get(action->pa_sync.out_params);
+        *action->pa_sync.out_params = broadcast_sink_periodic_sync_params;
+        break;
     case BLE_AUDIO_BROADCAST_SINK_ACTION_BIG_SYNC:
         console_printf("BIG Sync:\nsource_id=0x%02x iso_interval=0x%04x"
-                       " presentation_delay=%u[us]\n",
+                       " presentation_delay=%" PRIu32 "[us]\n",
                        action->big_sync.source_id, action->big_sync.iso_interval,
                        action->big_sync.presentation_delay);
         break;
@@ -503,20 +502,19 @@ broadcast_sink_action_fn(struct ble_audio_broadcast_sink_action *action, void *a
     return 0;
 }
 
-
+#if MYNEWT_VAL(SHELL_CMD_HELP)
 static const struct shell_param cmd_leaudio_broadcast_sink_start_params[] = {
     {"source_id", "usage: =<UINT8>"},
     {"broadcast_code", "usage: =[string], default: NULL"},
     {NULL, NULL}
 };
 
-#if MYNEWT_VAL(SHELL_CMD_HELP)
 const struct shell_cmd_help cmd_leaudio_broadcast_sink_start_help = {
     .summary = "Start audio Broadcast Sink",
     .usage = NULL,
     .params = cmd_leaudio_broadcast_sink_start_params
 };
-#endif
+#endif /* SHELL_CMD_HELP */
 
 int
 cmd_leaudio_broadcast_sink_start(int argc, char **argv)
@@ -551,18 +549,18 @@ cmd_leaudio_broadcast_sink_start(int argc, char **argv)
     return rc;
 }
 
+#if MYNEWT_VAL(SHELL_CMD_HELP)
 static const struct shell_param cmd_leaudio_broadcast_sink_stop_params[] = {
     {"source_id", "usage: =<UINT8>"},
     {NULL, NULL}
 };
 
-#if MYNEWT_VAL(SHELL_CMD_HELP)
 const struct shell_cmd_help cmd_leaudio_broadcast_sink_stop_help = {
     .summary = "Stop audio Broadcast Sink",
     .usage = NULL,
     .params = cmd_leaudio_broadcast_sink_stop_params
 };
-#endif
+#endif /* SHELL_CMD_HELP */
 
 int
 cmd_leaudio_broadcast_sink_stop(int argc, char **argv)
@@ -589,6 +587,7 @@ cmd_leaudio_broadcast_sink_stop(int argc, char **argv)
     return rc;
 }
 
+#if MYNEWT_VAL(SHELL_CMD_HELP)
 static const struct shell_param cmd_leaudio_broadcast_sink_metadata_update_params[] = {
     {"source_id", "usage: =<UINT8>"},
     {"subgroup_index", "usage: =<UINT8>"},
@@ -596,13 +595,12 @@ static const struct shell_param cmd_leaudio_broadcast_sink_metadata_update_param
     {NULL, NULL}
 };
 
-#if MYNEWT_VAL(SHELL_CMD_HELP)
 const struct shell_cmd_help cmd_leaudio_broadcast_sink_metadata_update_help = {
     .summary = "Update Broadcast Sink metadata",
     .usage = NULL,
     .params = cmd_leaudio_broadcast_sink_metadata_update_params
 };
-#endif
+#endif /* SHELL_CMD_HELP */
 
 int
 cmd_leaudio_broadcast_sink_metadata_update(int argc, char **argv)
@@ -647,6 +645,46 @@ cmd_leaudio_broadcast_sink_metadata_update(int argc, char **argv)
     return rc;
 }
 
+#if MYNEWT_VAL(SHELL_CMD_HELP)
+static const struct shell_param cmd_leaudio_broadcast_sink_sync_params_set_params[] = {
+    {"skip",         "usage: =[0x0000-0x01F3], default: 0x0000"},
+    {"sync_timeout", "usage: =[0x000A-0x4000], default: 0x07D0"},
+    {NULL, NULL}
+};
+
+const struct shell_cmd_help cmd_leaudio_broadcast_sink_sync_params_set_help = {
+    .summary = "Set Broadcast Sink sync parameters",
+    .usage = NULL,
+    .params = cmd_leaudio_broadcast_sink_sync_params_set_params
+};
+#endif /* SHELL_CMD_HELP */
+
+int
+cmd_leaudio_broadcast_sink_sync_params_set(int argc, char **argv)
+{
+    int rc;
+
+    rc = parse_arg_init(argc - 1, argv + 1);
+    if (rc != 0) {
+        return rc;
+    }
+
+    broadcast_sink_periodic_sync_params.skip = parse_arg_uint16_dflt("skip", BROADCAST_SINK_PA_SYNC_SKIP_DFLT, &rc);
+    if (rc != 0) {
+        console_printf("invalid 'skip' parameter\n");
+        return rc;
+    }
+
+    broadcast_sink_periodic_sync_params.sync_timeout = parse_arg_time_dflt("sync_timeout", 10000,
+                                                                           BROADCAST_SINK_PA_SYNC_TIMEOUT_DFLT, &rc);
+    if (rc != 0) {
+        console_printf("invalid 'sync_timeout' parameter\n");
+        return rc;
+    }
+
+    return rc;
+}
+
 static int
 broadcast_sink_audio_event_handler(struct ble_audio_event *event, void *arg)
 {
@@ -683,7 +721,7 @@ broadcast_sink_audio_event_handler(struct ble_audio_event *event, void *arg)
 static void
 scan_delegator_source_desc_printf(const struct ble_audio_scan_delegator_source_desc *source_desc)
 {
-    console_printf("broadcast_id=0x%6x adv_sid=%d adv_addr_type=%s adv_addr=",
+    console_printf("broadcast_id=0x%06" PRIx32 " adv_sid=%d adv_addr_type=%s adv_addr=",
                    source_desc->broadcast_id, source_desc->adv_sid,
                    cmd_addr_type_str(source_desc->addr.type));
     print_addr(source_desc->addr.val);
@@ -696,7 +734,7 @@ scan_delegator_sync_opt_printf(const struct ble_audio_scan_delegator_sync_opt *s
     console_printf("pa_sync=%d pa_interval=0x%04x num_subgroups=%d",
                    sync_opt->pa_sync, sync_opt->pa_interval, sync_opt->num_subgroups);
     for (uint8_t i = 0; i < sync_opt->num_subgroups; i++) {
-        console_printf("\n\tbis_sync=0x%04x metadata_length=%d metadata=",
+        console_printf("\n\tbis_sync=0x%04" PRIx32 " metadata_length=%d metadata=",
                        sync_opt->subgroups[i].bis_sync, sync_opt->subgroups[i].metadata_length);
         print_bytes(sync_opt->subgroups[i].metadata, sync_opt->subgroups[i].metadata_length);
     }
@@ -740,6 +778,7 @@ scan_delegator_action_fn(struct ble_audio_scan_delegator_action *action, void *a
     return 0;
 }
 
+#if MYNEWT_VAL(SHELL_CMD_HELP)
 static const struct shell_param cmd_leaudio_scan_delegator_receive_state_add_params[] = {
     {"addr_type", "usage: =[public|random], default: public"},
     {"addr", "usage: =[XX:XX:XX:XX:XX:XX]"},
@@ -748,7 +787,6 @@ static const struct shell_param cmd_leaudio_scan_delegator_receive_state_add_par
     {NULL, NULL}
 };
 
-#if MYNEWT_VAL(SHELL_CMD_HELP)
 const struct shell_cmd_help cmd_leaudio_scan_delegator_receive_state_add_help = {
     .summary = "Add receive state",
     .usage = NULL,
@@ -796,12 +834,12 @@ cmd_leaudio_scan_delegator_receive_state_add(int argc, char **argv)
     return rc;
 }
 
+#if MYNEWT_VAL(SHELL_CMD_HELP)
 static const struct shell_param cmd_leaudio_scan_delegator_receive_state_remove_params[] = {
     {"source_id", "usage: =<UINT8>"},
     {NULL, NULL}
 };
 
-#if MYNEWT_VAL(SHELL_CMD_HELP)
 const struct shell_cmd_help cmd_leaudio_scan_delegator_receive_state_remove_help = {
     .summary = "Remove receive state",
     .usage = NULL,
@@ -851,17 +889,15 @@ const struct parse_arg_kv_pair cmd_big_enc_type[] = {
     { NULL }
 };
 
+#if MYNEWT_VAL(SHELL_CMD_HELP)
 static const struct shell_param cmd_leaudio_scan_delegator_receive_state_set_params[] = {
     {"source_id", "usage: =<UINT8>"},
-    {"pa_sync_state", "usage: =[not_synced|sync_info_req|synced|failed|no_past],"
-     " default: not_synced"},
-    {"big_enc", "usage: =[not_encrypted|code_req|decrypting|bad_code],"
-     " default: not_encrypted"},
+    {"pa_sync_state", "usage: =[not_synced|sync_info_req|synced|failed|no_past], default: not_synced"},
+    {"big_enc", "usage: =[not_encrypted|code_req|decrypting|bad_code], default: not_encrypted"},
     {"bad_code", "usage: =[string], default: NULL"},
     {NULL, NULL}
 };
 
-#if MYNEWT_VAL(SHELL_CMD_HELP)
 const struct shell_cmd_help cmd_leaudio_scan_delegator_receive_state_set_help = {
     .summary = "Set receive state",
     .usage = NULL,
@@ -956,19 +992,19 @@ scan_delegator_receive_state_printf(const struct ble_audio_scan_delegator_receiv
     }
 
     for (uint8_t i = 0; i < state->num_subgroups; i++) {
-        console_printf("\n\tbis_sync=0x%04x metadata_length=%d metadata=",
+        console_printf("\n\tbis_sync=0x%04" PRIu32 " metadata_length=%d metadata=",
                        state->subgroups[i].bis_sync, state->subgroups[i].metadata_length);
         print_bytes(state->subgroups[i].metadata, state->subgroups[i].metadata_length);
         console_printf("\n");
     }
 }
 
+#if MYNEWT_VAL(SHELL_CMD_HELP)
 static const struct shell_param cmd_leaudio_scan_delegator_receive_state_get_params[] = {
     {"source_id", "usage: =<UINT8>"},
     {NULL, NULL}
 };
 
-#if MYNEWT_VAL(SHELL_CMD_HELP)
 const struct shell_cmd_help cmd_leaudio_scan_delegator_receive_state_get_help = {
     .summary = "Get receive state",
     .usage = NULL,
@@ -1044,7 +1080,7 @@ scan_delegator_audio_event_handler(struct ble_audio_event *event, void *arg)
     switch (event->type) {
     case BLE_AUDIO_EVENT_BROADCAST_ANNOUNCEMENT:
         console_printf("Broadcast Announcement\n");
-        console_printf("broadcast_id=0x%6x adv_sid=%d addr_type=%s addr=",
+        console_printf("broadcast_id=0x%06" PRIx32 " adv_sid=%d addr_type=%s addr=",
                        event->broadcast_announcement.broadcast_id,
                        event->broadcast_announcement.ext_disc->sid,
                        cmd_addr_type_str(event->broadcast_announcement.ext_disc->addr.type));
@@ -1086,3 +1122,4 @@ btshell_leaudio_init(void)
 #endif /* BLE_AUDIO_BROADCAST_SINK */
     assert(rc == 0);
 }
+#endif /* BLE_AUDIO */
