@@ -64,6 +64,8 @@ static ble_addr_t peer_id_addr;
 static ble_addr_t peer_ota_addr;
 static bool encrypted = false;
 
+static bool use_filter_policy = false;
+
 static struct os_callout                    update_params_co;
 static struct btp_gap_conn_param_update_cmd update_params;
 
@@ -126,55 +128,8 @@ supported_commands(const void *cmd, uint16_t cmd_len,
 {
     struct btp_gap_read_supported_commands_rp *rp = rsp;
 
-    /* octet 0 */
-    tester_set_bit(rp->data, BTP_GAP_READ_SUPPORTED_COMMANDS);
-    tester_set_bit(rp->data, BTP_GAP_READ_CONTROLLER_INDEX_LIST);
-    tester_set_bit(rp->data, BTP_GAP_READ_CONTROLLER_INFO);
-    tester_set_bit(rp->data, BTP_GAP_SET_CONNECTABLE);
-
-    /* octet 1 */
-    tester_set_bit(rp->data, BTP_GAP_SET_DISCOVERABLE);
-    tester_set_bit(rp->data, BTP_GAP_SET_BONDABLE);
-    tester_set_bit(rp->data, BTP_GAP_START_ADVERTISING);
-    tester_set_bit(rp->data, BTP_GAP_STOP_ADVERTISING);
-    tester_set_bit(rp->data, BTP_GAP_START_DISCOVERY);
-    tester_set_bit(rp->data, BTP_GAP_STOP_DISCOVERY);
-    tester_set_bit(rp->data, BTP_GAP_CONNECT);
-    tester_set_bit(rp->data, BTP_GAP_DISCONNECT);
-
-    /* octet 2 */
-    tester_set_bit(rp->data, BTP_GAP_SET_IO_CAP);
-    tester_set_bit(rp->data, BTP_GAP_PAIR);
-    tester_set_bit(rp->data, BTP_GAP_UNPAIR);
-    tester_set_bit(rp->data, BTP_GAP_PASSKEY_ENTRY);
-    tester_set_bit(rp->data, BTP_GAP_PASSKEY_CONFIRM);
-    tester_set_bit(rp->data, BTP_GAP_START_DIRECT_ADV);
-    tester_set_bit(rp->data, BTP_GAP_CONN_PARAM_UPDATE);
-
-    /* octet 3 */
-    tester_set_bit(rp->data, BTP_GAP_OOB_LEGACY_SET_DATA);
-    tester_set_bit(rp->data, BTP_GAP_OOB_SC_GET_LOCAL_DATA);
-    tester_set_bit(rp->data, BTP_GAP_OOB_SC_SET_REMOTE_DATA);
-    tester_set_bit(rp->data, BTP_GAP_SET_MITM);
-    tester_set_bit(rp->data, BTP_GAP_SET_FILTER_ACCEPT_LIST);
-
-    /* octet 4 */
-#if MYNEWT_VAL(BLE_PERIODIC_ADV)
-    tester_set_bit(rp->data, GAP_SET_EXT_ADV);
-    tester_set_bit(rp->data, GAP_PADV_CONFIGURE);
-    tester_set_bit(rp->data, GAP_PADV_START);
-    tester_set_bit(rp->data, GAP_PADV_SET_DATA);
-    tester_set_bit(rp->data, GAP_PADV_CREATE_SYNC);
-#endif
-#if MYNEWT_VAL(BLE_PERIODIC_ADV_SYNC_TRANSFER)
-    tester_set_bit(rp->data, GAP_PADV_SYNC_TRANSFER_SET_INFO);
-    tester_set_bit(rp->data, GAP_PADV_SYNC_TRANSFER_START);
-    tester_set_bit(rp->data, GAP_PADV_SYNC_TRANSFER_START);
-#endif
-
-    *rsp_len = sizeof(*rp) + 4 +
-               (MYNEWT_VAL(BLE_PERIODIC_ADV) ||
-                MYNEWT_VAL(BLE_PERIODIC_ADV_SYNC_TRANSFER) ? 1 : 0);
+    *rsp_len = tester_supported_commands(BTP_SERVICE_ID_GAP, rp->data);
+    *rsp_len += sizeof(*rp);
 
     return BTP_STATUS_SUCCESS;
 }
@@ -453,7 +408,7 @@ start_advertising(const void *cmd, uint16_t cmd_len,
     struct btp_gap_start_advertising_rp *rp = rsp;
     uint8_t buf[BLE_HS_ADV_MAX_SZ];
     uint8_t buf_len = 0;
-    uint8_t adv_len, sd_len;
+    uint8_t adv_len, sd_len = 0;
     uint8_t addr_type;
     uint32_t duration;
     int err;
@@ -496,7 +451,7 @@ start_advertising(const void *cmd, uint16_t cmd_len,
         i += ad[adv_len].data_len;
     }
 
-    for (sd_len = 0U; i < cp->scan_rsp_len; sd_len++) {
+    for (sd_len = 0U; i < (cp->adv_data_len + cp->scan_rsp_len); sd_len++) {
         if (sd_len >= ARRAY_SIZE(sd)) {
             SYS_LOG_ERR("sd[] Out of memory");
             return BTP_STATUS_FAILED;
@@ -515,6 +470,15 @@ start_advertising(const void *cmd, uint16_t cmd_len,
 
 #if MYNEWT_VAL(BLE_EXT_ADV)
     adv_params.own_addr_type = own_addr_type;
+
+    if (sd_len != 0 && adv_params.legacy_pdu) {
+        adv_params.scannable = 1;
+    }
+
+    if (use_filter_policy) {
+        adv_params.filter_policy = BLE_HCI_ADV_FILT_BOTH;
+    }
+
     err = ble_gap_ext_adv_configure(0, &adv_params, NULL, gap_event_cb, NULL);
     if (err) {
         SYS_LOG_ERR("Failed to configure extended advertiser; rc=%d", err);
@@ -531,6 +495,10 @@ start_advertising(const void *cmd, uint16_t cmd_len,
 #else
     err = ble_gap_adv_set_data(buf, buf_len);
 #endif
+    if (err) {
+        SYS_LOG_ERR("Failed to set advertising data; rc=%d", err);
+        return BTP_STATUS_FAILED;
+    }
 
     if (sd_len) {
         buf_len = 0;
@@ -1238,6 +1206,7 @@ sync_established(struct ble_gap_event *event)
 
     ev.status = event->periodic_sync.status;
     ev.sync_handle = event->periodic_sync.sync_handle;
+    ev.peer_addr = event->periodic_sync.adv_addr;
 
     tester_event(BTP_SERVICE_ID_GAP, GAP_EV_PERIODIC_SYNC_ESTABLISHED,
                  (uint8_t *) &ev, sizeof(ev));
@@ -1278,23 +1247,48 @@ periodic_report(struct ble_gap_event *event)
 static void
 periodic_transfer_received(struct ble_gap_event *event)
 {
+    int rc;
+    struct ble_gap_conn_desc desc;
     struct gap_periodic_transfer_recieved_ev ev;
 
-    ev.status = event->periodic_transfer.status;
-    ev.sync_handle = event->periodic_transfer.sync_handle;
-    ev.conn_handle = event->periodic_transfer.conn_handle;
-    ev.service_data = event->periodic_transfer.service_data;
-    ev.sid = event->periodic_transfer.sid;
     ev.adv_addr = event->periodic_transfer.adv_addr;
-    ev.adv_phy = event->periodic_transfer.adv_phy;
-    ev.per_adv_itvl = event->periodic_transfer.per_adv_itvl;
-    ev.adv_clk_accuracy = event->periodic_transfer.adv_clk_accuracy;
+    ev.sync_handle = event->periodic_transfer.sync_handle;
+    ev.status = event->periodic_transfer.status;
+
+    rc = ble_gap_conn_find(event->periodic_transfer.conn_handle, &desc);
+    assert(rc == 0);
+
+    ev.peer_addr = desc.peer_id_addr;
 
     tester_event(BTP_SERVICE_ID_GAP, GAP_EV_PERIODIC_TRANSFER_RECEIVED,
                  (uint8_t *) &ev, sizeof(ev));
 }
 #endif
 
+#if MYNEWT_VAL(BLE_CONN_SUBRATING)
+static void
+subrate_change_received(struct ble_gap_event *event)
+{
+    int rc;
+    struct ble_gap_conn_desc desc;
+    struct gap_subrate_change_ev ev;
+
+    rc = ble_gap_conn_find(event->subrate_change.conn_handle, &desc);
+    assert(rc == 0);
+
+    ev.addr = desc.peer_ota_addr;
+    ev.status = event->subrate_change.status;
+    ev.conn_handle = event->subrate_change.conn_handle;
+    ev.subrate_factor = event->subrate_change.subrate_factor;
+    ev.periph_latency = event->subrate_change.periph_latency;
+    ev.cont_num = event->subrate_change.cont_num;
+    ev.supervision_tmo = event->subrate_change.supervision_tmo;
+
+    tester_event(BTP_SERVICE_ID_GAP, GAP_EV_SUBRATE_CHANGE, (uint8_t *)&ev,
+                 sizeof(ev));
+}
+#endif
+
 static void
 print_bytes(const uint8_t *bytes, int len)
 {
@@ -1593,6 +1587,19 @@ gap_event_cb(struct ble_gap_event *event, void *arg)
                        event->periodic_transfer.adv_clk_accuracy);
         periodic_transfer_received(event);
         break;
+#endif
+#if MYNEWT_VAL(BLE_CONN_SUBRATING)
+    case BLE_GAP_EVENT_SUBRATE_CHANGE:
+        console_printf(
+            "Subrate change received:"
+            "status=%d, conn_handle=%d, subrate_factor=%d, perpih_latency=%d,"
+            "cont_num=%d supervision_tmo=%d",
+            event->subrate_change.status, event->subrate_change.conn_handle,
+            event->subrate_change.subrate_factor,
+            event->subrate_change.periph_latency, event->subrate_change.cont_num,
+            event->subrate_change.supervision_tmo);
+        subrate_change_received(event);
+        break;
 #endif
     default:
         break;
@@ -1984,6 +1991,8 @@ set_filter_accept_list(const void *cmd, uint16_t cmd_len,
 
     SYS_LOG_DBG("");
 
+    use_filter_policy = cp->list_len != 0;
+
     /*
      * Check if the nb of bytes received matches the len of addrs list.
      * Then set the filter accept list.
@@ -2238,6 +2247,29 @@ periodic_adv_sync_transfer_set_info(const void *cmd, uint16_t cmd_len,
 }
 #endif
 
+#if MYNEWT_VAL(BLE_CONN_SUBRATING)
+static uint8_t
+subrate_request(const void *cmd, uint16_t cmd_len, void *rsp, uint16_t *rsp_len)
+{
+    const struct gap_subrate_request_cmd *cp = cmd;
+    struct ble_gap_conn_desc desc;
+    int rc;
+
+    rc = gap_conn_find_by_addr(&cp->address, &desc);
+    if (rc) {
+        return BTP_STATUS_FAILED;
+    }
+
+    rc = ble_gap_subrate_req(desc.conn_handle, cp->subrate_min, cp->subrate_max,
+                             cp->max_latency, cp->cont_num, cp->supervision_timeout);
+    if (rc) {
+        return BTP_STATUS_FAILED;
+    }
+
+    return BTP_STATUS_SUCCESS;
+}
+#endif
+
 static const struct btp_handler handlers[] = {
     {
         .opcode = BTP_GAP_READ_SUPPORTED_COMMANDS,
@@ -2408,6 +2440,13 @@ static const struct btp_handler handlers[] = {
         .func = periodic_adv_sync_transfer_recv,
     },
 #endif
+#if MYNEWT_VAL(BLE_CONN_SUBRATING)
+    {
+        .opcode = GAP_SUBRATE_REQUEST,
+        .expect_len = sizeof(struct gap_subrate_request_cmd),
+        .func = subrate_request,
+    },
+#endif
 };
 
 static void
