@@ -1491,35 +1491,8 @@ supported_commands(const void *cmd, uint16_t cmd_len,
 {
     struct btp_gattc_read_supported_commands_rp *rp = rsp;
 
-    SYS_LOG_DBG("");
-
-    /* octet 0 */
-    tester_set_bit(rp->data, BTP_GATTC_READ_SUPPORTED_COMMANDS);
-    tester_set_bit(rp->data, BTP_GATTC_EXCHANGE_MTU);
-    tester_set_bit(rp->data, BTP_GATTC_DISC_ALL_PRIM_SVCS);
-    tester_set_bit(rp->data, BTP_GATTC_DISC_PRIM_UUID);
-    tester_set_bit(rp->data, BTP_GATTC_FIND_INCLUDED);
-    tester_set_bit(rp->data, BTP_GATTC_DISC_ALL_CHRC);
-    tester_set_bit(rp->data, BTP_GATTC_DISC_CHRC_UUID);
-    /* octet 1 */
-    tester_set_bit(rp->data, BTP_GATTC_DISC_ALL_DESC);
-    tester_set_bit(rp->data, BTP_GATTC_READ);
-    tester_set_bit(rp->data, BTP_GATTC_READ_UUID);
-    tester_set_bit(rp->data, BTP_GATTC_READ_LONG);
-    tester_set_bit(rp->data, BTP_GATTC_READ_MULTIPLE);
-    tester_set_bit(rp->data, BTP_GATTC_WRITE_WITHOUT_RSP);
-#if 0
-    tester_set_bit(rp->data, BTP_GATTC_SIGNED_WRITE_WITHOUT_RSP);
-#endif
-    tester_set_bit(rp->data, BTP_GATTC_WRITE);
-    /* octet 2 */
-    tester_set_bit(rp->data, BTP_GATTC_WRITE_LONG);
-    tester_set_bit(rp->data, BTP_GATTC_RELIABLE_WRITE);
-    tester_set_bit(rp->data, BTP_GATTC_CFG_NOTIFY);
-    tester_set_bit(rp->data, BTP_GATTC_CFG_INDICATE);
-    tester_set_bit(rp->data, BTP_GATTC_READ_MULTIPLE_VAR);
-
-    *rsp_len = sizeof(*rp) + 3;
+    *rsp_len = tester_supported_commands(BTP_SERVICE_ID_GATTC, rp->data);
+    *rsp_len += sizeof(*rp);
 
     return BTP_STATUS_SUCCESS;
 }
