@@ -285,6 +285,8 @@ extern STATS_SECT_DECL(ble_ll_stats) ble_ll_stats;
 #define BLE_LL_FEAT_CONN_SUBRATING      (0x0002000000000)
 #define BLE_LL_FEAT_CONN_SUBRATING_HOST (0x0004000000000)
 #define BLE_LL_FEAT_CHANNEL_CLASS       (0x0008000000000)
+#define BLE_LL_FEAT_ADV_CODING_SEL      (0x0010000000000)
+#define BLE_LL_FEAT_ADV_CODING_SEL_HOST (0x0020000000000)
 #define BLE_LL_FEAT_CS                  (0x0400000000000)
 #define BLE_LL_FEAT_CS_HOST_SUPPORT     (0x0800000000000)
 #define BLE_LL_FEAT_CS_PCT_QUALITY_IND  (0x1000000000000)
@@ -298,8 +300,9 @@ extern STATS_SECT_DECL(ble_ll_stats) ble_ll_stats;
 #define BLE_LL_CONN_CLEAR_FEATURE(connsm, feature)   (connsm->conn_features &= ~(feature))
 
 /* All the features which can be controlled by the Host */
-#if MYNEWT_VAL(BLE_LL_CFG_FEAT_LL_ENHANCED_CONN_UPDATE)
-#define BLE_LL_HOST_CONTROLLED_FEATURES (BLE_LL_FEAT_CONN_SUBRATING_HOST)
+#if MYNEWT_VAL(BLE_LL_CFG_FEAT_LL_ENHANCED_CONN_UPDATE) | \
+    MYNEWT_VAL(BLE_LL_ADV_CODING_SELECTION)
+#define BLE_LL_HOST_CONTROLLED_FEATURES (1)
 #else
 #define BLE_LL_HOST_CONTROLLED_FEATURES (0)
 #endif
