@@ -238,6 +238,9 @@ static inline int ble_ll_phy_to_phy_mode(int phy, int phy_options)
 #if MYNEWT_VAL(BLE_LL_DTM)
 void ble_phy_enable_dtm(void);
 void ble_phy_disable_dtm(void);
+#if MYNEWT_VAL(BLE_LL_DTM_EXTENSIONS)
+int ble_phy_dtm_carrier(uint8_t rf_channel);
+#endif
 #endif
 
 #ifdef __cplusplus
