@@ -46,8 +46,12 @@
 #include "controller/ble_ll_trace.h"
 #include "controller/ble_ll_sync.h"
 #include "controller/ble_fem.h"
-#include "controller/ble_ll_isoal.h"
+#if MYNEWT_VAL(BLE_LL_ISO)
+#include "controller/ble_ll_iso.h"
+#endif
+#if MYNEWT_VAL(BLE_LL_ISO_BROADCASTER)
 #include "controller/ble_ll_iso_big.h"
+#endif
 #if MYNEWT_VAL(BLE_LL_EXT)
 #include "controller/ble_ll_ext.h"
 #endif
@@ -82,6 +86,17 @@ static int8_t g_ble_ll_tx_power_phy_current;
 int8_t g_ble_ll_tx_power_compensation;
 int8_t g_ble_ll_rx_power_compensation;
 
+#if BLE_LL_HOST_CONTROLLED_FEATURES
+static const uint64_t g_ble_ll_host_controlled_features =
+#if MYNEWT_VAL(BLE_LL_CFG_FEAT_LL_ENHANCED_CONN_UPDATE)
+    BLE_LL_FEAT_CONN_SUBRATING_HOST |
+#endif
+#if MYNEWT_VAL(BLE_LL_ADV_CODING_SELECTION)
+    BLE_LL_FEAT_ADV_CODING_SEL_HOST |
+#endif
+    0;
+#endif
+
 /* Supported states */
 #if MYNEWT_VAL(BLE_LL_ROLE_BROADCASTER)
 #define BLE_LL_S_NCA                    ((uint64_t)1 << 0)
@@ -843,7 +858,7 @@ ble_ll_tx_pkt_in(void)
 
         /* Do some basic error checking */
         pb = handle & 0x3000;
-        if ((pkthdr->omp_len != length) || (pb > 0x1000) || (length == 0)) {
+        if ((pkthdr->omp_len != length) || (pb > 0x1000)) {
             /* This is a bad ACL packet. Count a stat and free it */
             STATS_INC(ble_ll_stats, bad_acl_hdr);
             os_mbuf_free_chain(om);
@@ -1460,6 +1475,7 @@ ble_ll_read_supp_features(void)
     return g_ble_ll_data.ll_supp_features;
 }
 
+#if BLE_LL_HOST_CONTROLLED_FEATURES
 /**
  * Sets the features controlled by the host.
  *
@@ -1486,7 +1502,7 @@ ble_ll_set_host_feat(const uint8_t *cmdbuf, uint8_t len)
     }
 
     mask = (uint64_t)1 << (cmd->bit_num);
-    if (!(mask & BLE_LL_HOST_CONTROLLED_FEATURES)) {
+    if (!(mask & g_ble_ll_host_controlled_features)) {
         return BLE_ERR_UNSUPPORTED;
     }
 
@@ -1498,6 +1514,8 @@ ble_ll_set_host_feat(const uint8_t *cmdbuf, uint8_t len)
 
     return BLE_ERR_SUCCESS;
 }
+#endif
+
 /**
  * Flush a link layer packet queue.
  *
@@ -1544,7 +1562,7 @@ ble_ll_mbuf_init(struct os_mbuf *m, uint8_t pdulen, uint8_t hdr)
 
     /* Set BLE transmit header */
     ble_hdr = BLE_MBUF_HDR_PTR(m);
-    ble_hdr->txinfo.flags = 0;
+    ble_hdr->txinfo.num_data_pkt = 0;
     ble_hdr->txinfo.offset = 0;
     ble_hdr->txinfo.pyld_len = pdulen;
     ble_hdr->txinfo.hdr_byte = hdr;
@@ -1678,13 +1696,14 @@ ble_ll_reset(void)
     ble_fem_lna_init();
 #endif
 
-#if MYNEWT_VAL(BLE_LL_ISO)
-    ble_ll_isoal_reset();
-#endif
 #if MYNEWT_VAL(BLE_LL_ISO_BROADCASTER)
     ble_ll_iso_big_reset();
 #endif
 
+#if MYNEWT_VAL(BLE_LL_ISO)
+    ble_ll_iso_reset();
+#endif
+
     /* Re-initialize the PHY */
     rc = ble_phy_init();
 
@@ -1939,6 +1958,10 @@ ble_ll_init(void)
     features |= BLE_LL_FEAT_PERIODIC_ADV_ADI;
 #endif
 
+#if MYNEWT_VAL(BLE_LL_ADV_CODING_SELECTION)
+    features |= BLE_LL_FEAT_ADV_CODING_SEL;
+#endif
+
     lldata->ll_supp_features = features;
 
     /* Initialize random number generation */
@@ -1961,7 +1984,7 @@ ble_ll_init(void)
 #endif
 
 #if MYNEWT_VAL(BLE_LL_ISO)
-    ble_ll_isoal_init();
+    ble_ll_iso_init();
 #endif
 #if MYNEWT_VAL(BLE_LL_ISO_BROADCASTER)
     ble_ll_iso_big_init();
