@@ -779,6 +779,9 @@ ble_ll_is_valid_adv_mode(uint8_t ocf)
 #if MYNEWT_VAL(BLE_VERSION) >= 51
     case BLE_HCI_OCF_LE_PERIODIC_ADV_RECEIVE_ENABLE:
 #endif
+#if MYNEWT_VAL(BLE_VERSION) >= 54
+    case BLE_HCI_OCF_LE_SET_EXT_ADV_PARAM_V2:
+#endif
 #if MYNEWT_VAL(BLE_LL_CFG_FEAT_LL_PERIODIC_ADV_SYNC_TRANSFER)
     case BLE_HCI_OCF_LE_PERIODIC_ADV_SYNC_TRANSFER:
     case BLE_HCI_OCF_LE_PERIODIC_ADV_SET_INFO_TRANSFER:
@@ -1281,16 +1284,22 @@ ble_ll_hci_le_cmd_proc(const uint8_t *cmdbuf, uint8_t len, uint16_t ocf,
     case BLE_HCI_OCF_LE_TERMINATE_BIG:
         rc = ble_ll_iso_big_hci_terminate(cmdbuf, len);
         break;
+    case BLE_HCI_OCF_LE_ISO_TRANSMIT_TEST:
+        rc = ble_ll_iso_transmit_test(cmdbuf, len, rspbuf, rsplen);
+        break;
 #endif /* BLE_LL_ISO_BROADCASTER */
 #if MYNEWT_VAL(BLE_LL_ISO)
     case BLE_HCI_OCF_LE_SETUP_ISO_DATA_PATH:
-        rc = ble_ll_isoal_hci_setup_iso_data_path(cmdbuf, len, rspbuf, rsplen);
+        rc = ble_ll_iso_setup_iso_data_path(cmdbuf, len, rspbuf, rsplen);
         break;
     case BLE_HCI_OCF_LE_REMOVE_ISO_DATA_PATH:
-        rc = ble_ll_isoal_hci_remove_iso_data_path(cmdbuf, len, rspbuf, rsplen);
+        rc = ble_ll_iso_remove_iso_data_path(cmdbuf, len, rspbuf, rsplen);
+        break;
+    case BLE_HCI_OCF_LE_ISO_TEST_END:
+        rc = ble_ll_iso_end_test(cmdbuf, len, rspbuf, rsplen);
         break;
     case BLE_HCI_OCF_LE_READ_ISO_TX_SYNC:
-        rc = ble_ll_isoal_hci_read_tx_sync(cmdbuf, len, rspbuf, rsplen);
+        rc = ble_ll_iso_read_tx_sync(cmdbuf, len, rspbuf, rsplen);
         break;
     case BLE_HCI_OCF_LE_RD_BUF_SIZE_V2:
         if (len == 0) {
@@ -1298,7 +1307,7 @@ ble_ll_hci_le_cmd_proc(const uint8_t *cmdbuf, uint8_t len, uint16_t ocf,
         }
         break;
 #endif /* BLE_LL_ISO */
-#if MYNEWT_VAL(BLE_VERSION) >= 52
+#if BLE_LL_HOST_CONTROLLED_FEATURES
     case BLE_HCI_OCF_LE_SET_HOST_FEATURE:
         rc = ble_ll_set_host_feat(cmdbuf, len);
         break;
@@ -1538,6 +1547,11 @@ ble_ll_hci_ctlr_bb_cmd_proc(const uint8_t *cmdbuf, uint8_t len, uint16_t ocf,
             rc = ble_ll_reset();
         }
         break;
+#if MYNEWT_VAL(BLE_LL_ROLE_PERIPHERAL) || MYNEWT_VAL(BLE_LL_ROLE_CENTRAL)
+    case BLE_HCI_OCF_CB_READ_TX_PWR:
+        rc = ble_ll_conn_hci_cb_read_tx_pwr(cmdbuf, len, rspbuf, rsplen);
+        break;
+#endif
 #if MYNEWT_VAL(BLE_LL_CFG_FEAT_CTRL_TO_HOST_FLOW_CONTROL)
     case BLE_HCI_OCF_CB_SET_CTLR_TO_HOST_FC:
         rc = ble_ll_hci_cb_set_ctrlr_to_host_fc(cmdbuf, len);
@@ -1735,16 +1749,21 @@ ble_ll_hci_cmd_fake_dual_mode(uint16_t opcode,  uint8_t *cmdbuf, uint8_t len,
         rc = 0;
         break;
     case BLE_HCI_OP(BLE_HCI_OGF_INFO_PARAMS, BLE_HCI_OCF_IP_RD_LOC_SUPP_FEAT):
-        put_le64(rspbuf, 0x877bffdbfe0ffebf);
+        put_le64(rspbuf, 0x077bffdbfe0ffebf);
         *rsplen = 8;
         rc = 0;
         break;
     case BLE_HCI_OP(BLE_HCI_OGF_INFO_PARAMS, 0x04): /* Read Local Extended Features */
-        rspbuf[0] = 0;
+        rspbuf[0] = cmdbuf[0];
         rspbuf[1] = 0;
-        put_le64(&rspbuf[2], 0x877bffdbfe0ffebf);
+        if (rspbuf[0] == 0) {
+            put_le64(&rspbuf[2], 0x077bffdbfe0ffebf);
+            rc = 0;
+        } else {
+            put_le64(&rspbuf[2], 0);
+            rc = BLE_ERR_INV_HCI_CMD_PARMS;
+        }
         *rsplen = 10;
-        rc = 0;
         break;
     case BLE_HCI_OP(BLE_HCI_OGF_INFO_PARAMS, BLE_HCI_OCF_IP_RD_BUF_SIZE):
         put_le16(rspbuf, 255);
@@ -1982,7 +2001,7 @@ int
 ble_ll_hci_iso_rx(struct os_mbuf *om)
 {
 #if MYNEWT_VAL(BLE_LL_ISO)
-    ble_ll_isoal_data_in(om);
+    ble_ll_iso_data_in(om);
 #else
     os_mbuf_free_chain(om);
 #endif
