@@ -39,6 +39,10 @@
 #error LE Coded PHY cannot be enabled on DA1469x
 #endif
 
+#ifndef min
+#define min(a, b) ((a) < (b) ? (a) : (b))
+#endif
+
 /* Statistics */
 STATS_SECT_START(ble_phy_stats)
     STATS_SECT_ENTRY(phy_isrs)
@@ -1361,8 +1365,6 @@ ble_phy_rx_set_start_time(uint32_t cputime, uint8_t rem_usecs)
     ble_phy_mode_apply(g_ble_phy_data.phy_mode_rx);
 #endif
 
-    assert(ble_rf_is_enabled());
-
     ble_phy_rx();
 
     /* Store LLT value at RX start, we'll need this to set wfr */
