@@ -7,7 +7,10 @@
  */
 
 #include "syscfg/syscfg.h"
-#define MESH_LOG_MODULE BLE_MESH_MODEL_LOG
+
+#define BLE_NPL_LOG_MODULE BLE_MESH_MODEL_LOG
+#include <nimble/nimble_npl_log.h>
+
 #if MYNEWT_VAL(BLE_MESH_CFG_CLI)
 
 #include "mesh/mesh.h"
@@ -462,7 +465,7 @@ static int mod_member_list_handle(struct bt_mesh_msg_ctx *ctx,
 				  struct os_mbuf *buf, bool vnd,
 				  struct mod_member_list_param *param)
 {
-	uint16_t elem_addr, mod_id, cid;
+	uint16_t elem_addr, mod_id, cid = 0;
 	uint8_t status;
 	int i;
 
