@@ -29,16 +29,13 @@
 #define BLE_GATTS_INCLUDE_SZ    6
 #define BLE_GATTS_CHR_MAX_SZ    19
 
-static const ble_uuid_t *uuid_pri =
-    BLE_UUID16_DECLARE(BLE_ATT_UUID_PRIMARY_SERVICE);
+static const ble_uuid_t *uuid_pri = BLE_UUID16_DECLARE(BLE_ATT_UUID_PRIMARY_SERVICE);
 static const ble_uuid_t *uuid_sec =
     BLE_UUID16_DECLARE(BLE_ATT_UUID_SECONDARY_SERVICE);
-static const ble_uuid_t *uuid_inc =
-    BLE_UUID16_DECLARE(BLE_ATT_UUID_INCLUDE);
-static const ble_uuid_t *uuid_chr =
-    BLE_UUID16_DECLARE(BLE_ATT_UUID_CHARACTERISTIC);
-static const ble_uuid_t *uuid_ccc =
-    BLE_UUID16_DECLARE(BLE_GATT_DSC_CLT_CFG_UUID16);
+static const ble_uuid_t *uuid_inc = BLE_UUID16_DECLARE(BLE_ATT_UUID_INCLUDE);
+static const ble_uuid_t *uuid_chr = BLE_UUID16_DECLARE(BLE_ATT_UUID_CHARACTERISTIC);
+static const ble_uuid_t *uuid_ccc = BLE_UUID16_DECLARE(BLE_GATT_DSC_CLT_CFG_UUID16);
+static const ble_uuid_t *uuid_cep = BLE_UUID16_DECLARE(BLE_GATT_DSC_EXT_PROP_UUID16);
 
 static const struct ble_gatt_svc_def **ble_gatts_svc_defs;
 static int ble_gatts_num_svc_defs;
@@ -154,6 +151,22 @@ ble_gatts_chr_clt_cfg_allowed(const struct ble_gatt_chr_def *chr)
     return flags;
 }
 
+static uint16_t
+ble_gatts_chr_ext_prop_allowed(const struct ble_gatt_chr_def *chr)
+{
+    uint16_t flags;
+
+    flags = 0;
+    if (chr->flags & BLE_GATT_CHR_F_RELIABLE_WRITE) {
+        flags |= BLE_GATTS_CEP_F_RELIABLE_WRITE;
+    }
+    if (chr->flags & BLE_GATT_CHR_F_AUX_WRITE) {
+        flags |= BLE_GATTS_CEP_F_AUX_WRITE;
+    }
+
+    return flags;
+}
+
 static uint8_t
 ble_gatts_att_flags_from_chr_flags(ble_gatt_chr_flags chr_flags)
 {
@@ -416,6 +429,7 @@ ble_gatts_chr_val_access(uint16_t conn_handle, uint16_t attr_handle,
 
     gatt_ctxt.op = ble_gatts_chr_op(att_op);
     gatt_ctxt.chr = chr_def;
+    gatt_ctxt.offset = offset;
 
     ble_gatts_chr_inc_val_stat(gatt_ctxt.op);
     rc = ble_gatts_val_access(conn_handle, attr_handle, offset, &gatt_ctxt, om,
@@ -736,6 +750,33 @@ ble_gatts_clt_cfg_access_locked(struct ble_hs_conn *conn, uint16_t attr_handle,
     return 0;
 }
 
+int
+ble_gatts_read_cccd(uint16_t conn_handle, uint16_t chr_val_handle, uint8_t *cccd_value)
+{
+    struct ble_gatts_clt_cfg *clt_cfg;
+    struct ble_hs_conn *conn;
+
+    ble_hs_lock();
+
+    conn = ble_hs_conn_find(conn_handle);
+    if (conn == NULL) {
+        ble_hs_unlock();
+        return BLE_HS_ENOTCONN;
+    }
+
+    clt_cfg = ble_gatts_clt_cfg_find(conn->bhc_gatt_svr.clt_cfgs, chr_val_handle);
+    if (clt_cfg == NULL) {
+        ble_hs_unlock();
+        return BLE_HS_ENOENT;
+    }
+
+    *cccd_value = clt_cfg->flags & ~BLE_GATTS_CLT_CFG_F_RESERVED;
+
+    ble_hs_unlock();
+
+    return 0;
+}
+
 int
 ble_gatts_clt_cfg_access(uint16_t conn_handle, uint16_t attr_handle,
                          uint8_t op, uint16_t offset, struct os_mbuf **om,
@@ -803,6 +844,55 @@ ble_gatts_register_clt_cfg_dsc(uint16_t *att_handle)
     return 0;
 }
 
+static int
+ble_gatts_cep_access(uint16_t conn_handle, uint16_t attr_handle, uint8_t att_op,
+                     uint16_t offset, struct os_mbuf **om, void *arg)
+{
+    uint16_t prop = POINTER_TO_UINT(arg);
+    uint8_t *buf;
+
+    ble_gatts_dsc_inc_stat(ble_gatts_dsc_op(att_op));
+
+    if (att_op != BLE_ATT_ACCESS_OP_READ) {
+        return BLE_ATT_ERR_WRITE_NOT_PERMITTED;
+    }
+
+    buf = os_mbuf_extend(*om, sizeof(prop));
+    if (buf == NULL) {
+        return BLE_ATT_ERR_INSUFFICIENT_RES;
+    }
+    put_le16(buf, prop);
+
+    return 0;
+}
+
+static int
+ble_gatts_register_cep_dsc(uint16_t *att_handle, ble_gatt_chr_flags flags)
+{
+    struct ble_gatt_cep_dsc cep;
+    int rc;
+
+    cep.properties = 0x0000;
+    if (flags & BLE_GATT_CHR_F_RELIABLE_WRITE) {
+        cep.properties |= BLE_GATTS_CEP_F_RELIABLE_WRITE;
+    }
+    if (flags & BLE_GATT_CHR_F_AUX_WRITE) {
+        /* TODO: Implement Characteristic User Description
+         * (Core specification 6.0, vol 3, part G, section 3.3.3.2)*/
+        cep.properties |= BLE_GATTS_CEP_F_AUX_WRITE;
+    }
+
+    rc = ble_att_svr_register(uuid_cep, BLE_ATT_F_READ, 0, att_handle,
+                              ble_gatts_cep_access, UINT_TO_POINTER(cep.properties));
+    if (rc != 0) {
+        return rc;
+    }
+
+    STATS_INC(ble_gatts_stats, dscs);
+
+    return 0;
+}
+
 static int
 ble_gatts_register_chr(const struct ble_gatt_svc_def *svc,
                        const struct ble_gatt_chr_def *chr,
@@ -813,6 +903,8 @@ ble_gatts_register_chr(const struct ble_gatt_svc_def *svc,
     uint16_t def_handle;
     uint16_t val_handle;
     uint16_t dsc_handle;
+    uint16_t cep_handle;
+    uint16_t expected_handle;
     uint8_t att_flags;
     int rc;
 
@@ -836,6 +928,9 @@ ble_gatts_register_chr(const struct ble_gatt_svc_def *svc,
         return rc;
     }
 
+    expected_handle = def_handle;
+    (void)expected_handle;
+
     /* Register characteristic value attribute (cast away const on callback
      * arg).
      */
@@ -846,7 +941,7 @@ ble_gatts_register_chr(const struct ble_gatt_svc_def *svc,
     if (rc != 0) {
         return rc;
     }
-    BLE_HS_DBG_ASSERT(val_handle == def_handle + 1);
+    BLE_HS_DBG_ASSERT(val_handle == ++expected_handle);
 
     if (chr->val_handle != NULL) {
         *chr->val_handle = val_handle;
@@ -866,7 +961,15 @@ ble_gatts_register_chr(const struct ble_gatt_svc_def *svc,
         if (rc != 0) {
             return rc;
         }
-        BLE_HS_DBG_ASSERT(dsc_handle == def_handle + 2);
+        BLE_HS_DBG_ASSERT(dsc_handle == ++expected_handle);
+    }
+
+    if (ble_gatts_chr_ext_prop_allowed(chr) != 0) {
+        rc = ble_gatts_register_cep_dsc(&cep_handle, chr->flags);
+        if (rc != 0) {
+            return rc;
+        }
+        BLE_HS_DBG_ASSERT(cep_handle == ++expected_handle);
     }
 
     /* Register each descriptor. */
@@ -1618,8 +1721,6 @@ ble_gatts_peer_cl_sup_feat_update(uint16_t conn_handle, struct os_mbuf *om)
     int rc = 0;
     int i;
 
-    BLE_HS_LOG(DEBUG, "");
-
     if (!om) {
         return BLE_ATT_ERR_INSUFFICIENT_RES;
     }
@@ -2173,6 +2274,21 @@ ble_gatts_count_resources(const struct ble_gatt_svc_def *svcs,
                     res->attrs++;
                 }
 
+                /* If the characteristic permits reliable writes or auxiliary
+                 * writes, it has an Extended Properties descriptor.
+                 */
+                if (chr->flags & BLE_GATT_CHR_F_AUX_WRITE ||
+                    chr->flags & BLE_GATT_CHR_F_RELIABLE_WRITE) {
+
+                    /* Each CEP requires:
+                     *     o 1 descriptor
+                     *     o 1 attribute
+                     */
+                    res->dscs++;
+                    res->ceps++;
+                    res->attrs++;
+                }
+
                 if (chr->descriptors != NULL) {
                     for (d = 0; chr->descriptors[d].uuid != NULL; d++) {
                         if (!ble_gatts_dsc_is_sane(chr->descriptors + d)) {
