@@ -25,8 +25,20 @@
 #include "ble_l2cap_coc_priv.h"
 #include "ble_l2cap_sig_priv.h"
 
+#ifndef min
+#define min(a, b) ((a) < (b) ? (a) : (b))
+#endif
+
 #if MYNEWT_VAL(BLE_L2CAP_COC_MAX_NUM) != 0 && NIMBLE_BLE_CONNECT
 
+#ifndef min
+#define min(a, b) ((a) < (b) ? (a) : (b))
+#endif
+
+#ifndef max
+#define max(a, b) ((a) > (b) ? (a) : (b))
+#endif
+
 #define BLE_L2CAP_SDU_SIZE              2
 
 STAILQ_HEAD(ble_l2cap_coc_srv_list, ble_l2cap_coc_srv);
@@ -40,18 +52,6 @@ static os_membuf_t ble_l2cap_coc_srv_mem[
 
 static struct os_mempool ble_l2cap_coc_srv_pool;
 
-static void
-ble_l2cap_coc_dbg_assert_srv_not_inserted(struct ble_l2cap_coc_srv *srv)
-{
-#if MYNEWT_VAL(BLE_HS_DEBUG)
-    struct ble_l2cap_coc_srv *cur;
-
-    STAILQ_FOREACH(cur, &ble_l2cap_coc_srvs, next) {
-        BLE_HS_DBG_ASSERT(cur != srv);
-    }
-#endif
-}
-
 static struct ble_l2cap_coc_srv *
 ble_l2cap_coc_srv_alloc(void)
 {
@@ -65,12 +65,31 @@ ble_l2cap_coc_srv_alloc(void)
     return srv;
 }
 
-int
-ble_l2cap_coc_create_server(uint16_t psm, uint16_t mtu,
-                            ble_l2cap_event_fn *cb, void *cb_arg)
+static struct ble_l2cap_coc_srv *
+ble_l2cap_coc_srv_find(uint16_t psm)
 {
     struct ble_l2cap_coc_srv *srv;
 
+    STAILQ_FOREACH(srv, &ble_l2cap_coc_srvs, next) {
+        if (srv->psm == psm) {
+            return srv;
+        }
+    }
+
+    return NULL;
+}
+
+int
+ble_l2cap_coc_create_server_nolock(uint16_t psm, uint16_t mtu,
+                                   ble_l2cap_event_fn *cb, void *cb_arg)
+{
+    struct ble_l2cap_coc_srv *srv;
+
+    srv = ble_l2cap_coc_srv_find(psm);
+    if (srv) {
+        return BLE_HS_EALREADY;
+    }
+
     srv = ble_l2cap_coc_srv_alloc();
     if (!srv) {
         return BLE_HS_ENOMEM;
@@ -81,13 +100,27 @@ ble_l2cap_coc_create_server(uint16_t psm, uint16_t mtu,
     srv->cb = cb;
     srv->cb_arg = cb_arg;
 
-    ble_l2cap_coc_dbg_assert_srv_not_inserted(srv);
-
     STAILQ_INSERT_HEAD(&ble_l2cap_coc_srvs, srv, next);
 
     return 0;
 }
 
+int
+ble_l2cap_coc_remove_server_nolock(uint16_t psm)
+{
+    struct ble_l2cap_coc_srv *srv;
+
+    srv = ble_l2cap_coc_srv_find(psm);
+    if (!srv) {
+        return BLE_HS_ENOENT;
+    }
+
+    STAILQ_REMOVE(&ble_l2cap_coc_srvs, srv, ble_l2cap_coc_srv, next);
+
+    os_memblock_put(&ble_l2cap_coc_srv_pool, srv);
+    return 0;
+}
+
 static inline void
 ble_l2cap_set_used_cid(uint32_t *cid_mask, int bit)
 {
@@ -140,22 +173,6 @@ ble_l2cap_coc_get_cid(uint32_t *cid_mask)
     return BLE_L2CAP_COC_CID_START + bit;
 }
 
-static struct ble_l2cap_coc_srv *
-ble_l2cap_coc_srv_find(uint16_t psm)
-{
-    struct ble_l2cap_coc_srv *cur, *srv;
-
-    srv = NULL;
-    STAILQ_FOREACH(cur, &ble_l2cap_coc_srvs, next) {
-        if (cur->psm == psm) {
-            srv = cur;
-            break;
-        }
-    }
-
-    return srv;
-}
-
 static void
 ble_l2cap_event_coc_received_data(struct ble_l2cap_chan *chan,
                                   struct os_mbuf *om)
@@ -171,29 +188,21 @@ ble_l2cap_event_coc_received_data(struct ble_l2cap_chan *chan,
 }
 
 static int
-ble_l2cap_coc_rx_fn(struct ble_l2cap_chan *chan)
+ble_l2cap_coc_rx_fn(struct ble_l2cap_chan *chan, struct os_mbuf **om)
 {
-    int rc;
-    struct os_mbuf **om;
+    struct ble_l2cap_coc_endpoint *rx = &chan->coc_rx;
     struct os_mbuf *rx_sdu;
-    struct ble_l2cap_coc_endpoint *rx;
+    uint16_t sdu_idx;
     uint16_t om_total;
+    int rc;
 
-    /* Create a shortcut to rx_buf */
-    om = &chan->rx_buf;
-    BLE_HS_DBG_ASSERT(*om != NULL);
-
-    /* Create a shortcut to rx endpoint */
-    rx = &chan->coc_rx;
-    BLE_HS_DBG_ASSERT(rx != NULL);
-
-    rx_sdu = rx->sdus[chan->coc_rx.current_sdu_idx];
-    BLE_HS_DBG_ASSERT(rx_sdu != NULL);
+    sdu_idx = chan->coc_rx.current_sdu_idx;
+    BLE_HS_DBG_ASSERT(rx->sdus[sdu_idx] != NULL);
 
     om_total = OS_MBUF_PKTLEN(*om);
 
     /* First LE frame */
-    if (OS_MBUF_PKTLEN(rx_sdu) == 0) {
+    if (OS_MBUF_PKTLEN(rx->sdus[sdu_idx]) == 0) {
         uint16_t sdu_len;
 
         rc = ble_hs_mbuf_pullup_base(om, BLE_L2CAP_SDU_SIZE);
@@ -211,7 +220,6 @@ ble_l2cap_coc_rx_fn(struct ble_l2cap_chan *chan)
             BLE_HS_LOG(ERROR, "Payload larger than expected (%d>%d)\n",
                        om_total, sdu_len + 2);
             /* Disconnect peer with invalid behaviour */
-            rx_sdu = NULL;
             rx->data_offset = 0;
             ble_l2cap_disconnect(chan);
             return BLE_HS_EBADDATA;
@@ -231,12 +239,13 @@ ble_l2cap_coc_rx_fn(struct ble_l2cap_chan *chan)
 
         os_mbuf_adj(*om, BLE_L2CAP_SDU_SIZE);
 
-        rc = os_mbuf_appendfrom(rx_sdu, *om, 0, om_total - BLE_L2CAP_SDU_SIZE);
+        rc = os_mbuf_appendfrom(rx->sdus[sdu_idx], *om, 0,
+                                om_total - BLE_L2CAP_SDU_SIZE);
         if (rc != 0) {
             /* FIXME: User shall give us big enough buffer.
              * need to handle it better
              */
-            BLE_HS_LOG(INFO, "Could not append data rc=%d\n", rc);
+            BLE_HS_LOG(ERROR, "Could not append data rc=%d\n", rc);
             assert(0);
         }
 
@@ -246,16 +255,16 @@ ble_l2cap_coc_rx_fn(struct ble_l2cap_chan *chan)
     } else {
         BLE_HS_LOG(DEBUG, "Continuation...received %d\n", (*om)->om_len);
 
-        if (OS_MBUF_PKTLEN(rx_sdu) + (*om)->om_len > rx->data_offset) {
+        if (OS_MBUF_PKTLEN(rx->sdus[sdu_idx]) + (*om)->om_len > rx->data_offset) {
             /* Disconnect peer with invalid behaviour */
             BLE_HS_LOG(ERROR, "Payload larger than expected (%d>%d)\n",
-                       OS_MBUF_PKTLEN(rx_sdu) + (*om)->om_len, rx->data_offset);
-            rx_sdu = NULL;
+                       OS_MBUF_PKTLEN(rx->sdus[sdu_idx]) + (*om)->om_len,
+                       rx->data_offset);
             rx->data_offset = 0;
             ble_l2cap_disconnect(chan);
             return BLE_HS_EBADDATA;
         }
-        rc = os_mbuf_appendfrom(rx_sdu, *om, 0, om_total);
+        rc = os_mbuf_appendfrom(rx->sdus[sdu_idx], *om, 0, om_total);
         if (rc != 0) {
             /* FIXME: need to handle it better */
             BLE_HS_LOG(DEBUG, "Could not append data rc=%d\n", rc);
@@ -265,22 +274,22 @@ ble_l2cap_coc_rx_fn(struct ble_l2cap_chan *chan)
 
     rx->credits--;
 
-    if (OS_MBUF_PKTLEN(rx_sdu) == rx->data_offset) {
-        struct os_mbuf *sdu_rx = rx_sdu;
-
+    if (OS_MBUF_PKTLEN(rx->sdus[sdu_idx]) == rx->data_offset) {
         BLE_HS_LOG(DEBUG, "Received sdu_len=%d, credits left=%d\n",
-                   OS_MBUF_PKTLEN(rx_sdu), rx->credits);
+                   OS_MBUF_PKTLEN(rx->sdus[sdu_idx]), rx->credits);
 
-        /* Lets get back control to os_mbuf to application.
-         * Since it this callback application might want to set new sdu
-         * we need to prepare space for this. Therefore we need sdu_rx
-         */
-        rx_sdu = NULL;
         chan->coc_rx.current_sdu_idx =
             (chan->coc_rx.current_sdu_idx + 1) % BLE_L2CAP_SDU_BUFF_CNT;
         rx->data_offset = 0;
 
-        ble_l2cap_event_coc_received_data(chan, sdu_rx);
+        /* Lets give os_mbuf control to back application.
+         * Since it this callback application might want to set new sdu
+         * we need to prepare space for this.
+         */
+        rx_sdu = rx->sdus[sdu_idx];
+        rx->sdus[sdu_idx] = NULL;
+
+        ble_l2cap_event_coc_received_data(chan, rx_sdu);
 
         return 0;
     }
@@ -298,9 +307,9 @@ ble_l2cap_coc_rx_fn(struct ble_l2cap_chan *chan)
         ble_l2cap_sig_le_credits(chan->conn_handle, chan->scid, rx->credits);
     }
 
-    BLE_HS_LOG(DEBUG,
-               "Received partial sdu_len=%d, credits left=%d, current_sdu_idx=%d\n",
-               OS_MBUF_PKTLEN(rx_sdu), rx->credits, chan->coc_rx.current_sdu_idx);
+    BLE_HS_LOG(DEBUG, "Received partial sdu_len=%d, credits left=%d, current_sdu_idx=%d\n",
+               OS_MBUF_PKTLEN(rx->sdus[sdu_idx]), rx->credits,
+               chan->coc_rx.current_sdu_idx);
 
     return 0;
 }
@@ -592,8 +601,8 @@ ble_l2cap_coc_le_credits_update(uint16_t conn_handle, uint16_t dcid,
 
     if (chan->coc_tx.credits + credits > 0xFFFF) {
         BLE_HS_LOG(INFO, "LE CoC credits overflow...disconnecting\n");
+        ble_l2cap_sig_disconnect_nolock(chan);
         ble_hs_unlock();
-        ble_l2cap_sig_disconnect(chan);
         return;
     }
 
@@ -641,10 +650,8 @@ ble_l2cap_coc_recv_ready(struct ble_l2cap_chan *chan, struct os_mbuf *sdu_rx)
      * to be able to send complete SDU.
      */
     if (chan->coc_rx.credits < c->initial_credits) {
-        ble_hs_unlock();
-        ble_l2cap_sig_le_credits(chan->conn_handle, chan->scid,
-                                 c->initial_credits - chan->coc_rx.credits);
-        ble_hs_lock();
+        ble_l2cap_sig_le_credits_nolock(chan->conn_handle, chan->scid,
+                                        c->initial_credits - chan->coc_rx.credits);
         chan->coc_rx.credits = c->initial_credits;
     }
 
