@@ -248,16 +248,6 @@ ble_l2cap_sig_proc_free(struct ble_l2cap_sig_proc *proc)
     }
 }
 
-static void
-ble_l2cap_sig_proc_insert(struct ble_l2cap_sig_proc *proc)
-{
-    ble_l2cap_sig_dbg_assert_proc_not_inserted(proc);
-
-    ble_hs_lock();
-    STAILQ_INSERT_HEAD(&ble_l2cap_sig_procs, proc, next);
-    ble_hs_unlock();
-}
-
 /**
  * Tests if a proc entry fits the specified criteria.
  *
@@ -345,14 +335,20 @@ ble_l2cap_sig_proc_set_timer(struct ble_l2cap_sig_proc *proc)
 }
 
 static void
-ble_l2cap_sig_process_status(struct ble_l2cap_sig_proc *proc, int status)
+ble_l2cap_sig_proc_insert(struct ble_l2cap_sig_proc *proc)
 {
-    if (status == 0) {
-        ble_l2cap_sig_proc_set_timer(proc);
-        ble_l2cap_sig_proc_insert(proc);
-    } else {
-        ble_l2cap_sig_proc_free(proc);
-    }
+    BLE_HS_DBG_ASSERT(ble_hs_locked_by_cur_task());
+
+    ble_l2cap_sig_dbg_assert_proc_not_inserted(proc);
+
+    STAILQ_INSERT_HEAD(&ble_l2cap_sig_procs, proc, next);
+}
+
+static void
+ble_l2cap_sig_proc_start(struct ble_l2cap_sig_proc *proc)
+{
+    ble_l2cap_sig_proc_set_timer(proc);
+    ble_l2cap_sig_proc_insert(proc);
 }
 
 /*****************************************************************************
@@ -407,6 +403,38 @@ ble_l2cap_sig_check_conn_params(const struct ble_gap_upd_params *params)
     return 0;
 }
 
+/* This helper does validation of sizes and (if requested) CIDs sizes */
+static int
+ble_l2cap_sig_mbuf_pullup_base(struct os_mbuf **om, int base_len, unsigned int *cids_count)
+{
+    if (cids_count) {
+        if (OS_MBUF_PKTLEN(*om) < base_len) {
+            return BLE_HS_EBADDATA;
+        }
+
+        if ((OS_MBUF_PKTLEN(*om) - base_len) % sizeof(uint16_t)) {
+            return BLE_HS_EBADDATA;
+        }
+
+        *cids_count = (OS_MBUF_PKTLEN(*om) - base_len) / sizeof(uint16_t);
+        if (*cids_count == 0 || *cids_count > BLE_L2CAP_MAX_COC_CONN_REQ) {
+            return BLE_HS_EBADDATA;
+        }
+    } else {
+        if (OS_MBUF_PKTLEN(*om) != base_len) {
+            return BLE_HS_EBADDATA;
+        }
+    }
+
+    /* data sizes validated, just pullup all */
+    *om = os_mbuf_pullup(*om, OS_MBUF_PKTLEN(*om));
+    if (*om == NULL) {
+        return BLE_HS_ENOMEM;
+    }
+
+    return 0;
+}
+
 int
 ble_l2cap_sig_update_req_rx(uint16_t conn_handle,
                             struct ble_l2cap_sig_hdr *hdr,
@@ -423,7 +451,7 @@ ble_l2cap_sig_update_req_rx(uint16_t conn_handle,
 
     l2cap_result = 0; /* Silence spurious gcc warning. */
 
-    rc = ble_hs_mbuf_pullup_base(om, BLE_L2CAP_SIG_UPDATE_REQ_SZ);
+    rc = ble_l2cap_sig_mbuf_pullup_base(om, sizeof(*req), NULL);
     if (rc != 0) {
         return rc;
     }
@@ -500,7 +528,7 @@ ble_l2cap_sig_update_rsp_rx(uint16_t conn_handle,
         return 0;
     }
 
-    rc = ble_hs_mbuf_pullup_base(om, BLE_L2CAP_SIG_UPDATE_RSP_SZ);
+    rc = ble_l2cap_sig_mbuf_pullup_base(om, sizeof(*rsp), NULL);
     if (rc != 0) {
         cb_status = rc;
         goto done;
@@ -532,9 +560,9 @@ done:
 }
 
 int
-ble_l2cap_sig_update(uint16_t conn_handle,
-                     struct ble_l2cap_sig_update_params *params,
-                     ble_l2cap_sig_update_fn *cb, void *cb_arg)
+ble_l2cap_sig_update_nolock(uint16_t conn_handle,
+                            struct ble_l2cap_sig_update_params *params,
+                            ble_l2cap_sig_update_fn *cb, void *cb_arg)
 {
     struct os_mbuf *txom;
     struct ble_l2cap_sig_update_req *req;
@@ -548,30 +576,25 @@ ble_l2cap_sig_update(uint16_t conn_handle,
 
     STATS_INC(ble_l2cap_stats, update_init);
 
-    ble_hs_lock();
     rc = ble_hs_misc_conn_chan_find_reqd(conn_handle, BLE_L2CAP_CID_SIG,
                                          &conn, &chan);
     if (rc != 0) {
-        ble_hs_unlock();
-        goto done;
+        return rc;
     }
 
     master = conn->bhc_flags & BLE_HS_CONN_F_MASTER;
-    ble_hs_unlock();
 
     if (master) {
         /* Only the slave can initiate the L2CAP connection update
          * procedure.
          */
-        rc = BLE_HS_EINVAL;
-        goto done;
+        return BLE_HS_EINVAL;
     }
 
     proc = ble_l2cap_sig_proc_alloc();
     if (proc == NULL) {
         STATS_INC(ble_l2cap_stats, update_fail);
-        rc = BLE_HS_ENOMEM;
-        goto done;
+        return BLE_HS_ENOMEM;
     }
 
     proc->op = BLE_L2CAP_SIG_PROC_OP_UPDATE;
@@ -584,8 +607,8 @@ ble_l2cap_sig_update(uint16_t conn_handle,
                                 sizeof(*req), &txom);
     if (!req) {
         STATS_INC(ble_l2cap_stats, update_fail);
-        rc = BLE_HS_ENOMEM;
-        goto done;
+        ble_l2cap_sig_proc_free(proc);
+        return BLE_HS_ENOMEM;
     }
 
     req->itvl_min = htole16(params->itvl_min);
@@ -593,10 +616,27 @@ ble_l2cap_sig_update(uint16_t conn_handle,
     req->slave_latency = htole16(params->slave_latency);
     req->timeout_multiplier = htole16(params->timeout_multiplier);
 
-    rc = ble_l2cap_sig_tx(conn_handle, txom);
+    rc = ble_l2cap_sig_tx_nolock(conn_handle, txom);
+    if (rc) {
+        STATS_INC(ble_l2cap_stats, update_fail);
+        ble_l2cap_sig_proc_free(proc);
+        return rc;
+    }
+
+    ble_l2cap_sig_proc_start(proc);
+    return 0;
+}
+
+int
+ble_l2cap_sig_update(uint16_t conn_handle, struct ble_l2cap_sig_update_params *params,
+                     ble_l2cap_sig_update_fn *cb, void *cb_arg)
+{
+    int rc;
+
+    ble_hs_lock();
+    rc = ble_l2cap_sig_update_nolock(conn_handle, params, cb, cb_arg);
+    ble_hs_unlock();
 
-done:
-    ble_l2cap_sig_process_status(proc, rc);
     return rc;
 }
 
@@ -766,12 +806,12 @@ ble_l2cap_sig_credit_base_reconfig_req_rx(uint16_t conn_handle,
     struct ble_l2cap_sig_credit_base_reconfig_rsp *rsp;
     struct ble_hs_conn *conn;
     struct os_mbuf *txom;
-    int i;
+    unsigned int i;
     int rc;
-    uint8_t cid_cnt;
+    unsigned int cid_cnt;
     uint8_t reduction_mps = 0;
 
-    rc = ble_hs_mbuf_pullup_base(om, hdr->length);
+    rc = ble_l2cap_sig_mbuf_pullup_base(om, sizeof(*req), &cid_cnt);
     if (rc != 0) {
         return rc;
     }
@@ -794,11 +834,6 @@ ble_l2cap_sig_credit_base_reconfig_req_rx(uint16_t conn_handle,
         return 0;
     }
 
-    if (hdr->length <= sizeof(*req)) {
-        rsp->result = htole16(BLE_L2CAP_ERR_RECONFIG_UNACCEPTED_PARAM);
-        goto failed;
-    }
-
     req = (struct ble_l2cap_sig_credit_base_reconfig_req *)(*om)->om_data;
 
     if ((req->mps < BLE_L2CAP_ECOC_MIN_MTU) || (req->mtu < BLE_L2CAP_ECOC_MIN_MTU)) {
@@ -809,12 +844,6 @@ ble_l2cap_sig_credit_base_reconfig_req_rx(uint16_t conn_handle,
     /* Assume request will succeed. If not, result will be updated */
     rsp->result = htole16(BLE_L2CAP_ERR_RECONFIG_SUCCEED);
 
-    cid_cnt = (hdr->length - sizeof(*req)) / sizeof(uint16_t);
-    if (cid_cnt > BLE_L2CAP_MAX_COC_CONN_REQ) {
-        rsp->result = htole16(BLE_L2CAP_ERR_RECONFIG_UNACCEPTED_PARAM);
-        goto failed;
-    }
-
     for (i = 0; i < cid_cnt; i++) {
         chan[i] = ble_hs_conn_chan_find_by_dcid(conn, req->dcids[i]);
         if (!chan[i]) {
@@ -837,6 +866,8 @@ ble_l2cap_sig_credit_base_reconfig_req_rx(uint16_t conn_handle,
         goto failed;
     }
 
+    ble_l2cap_sig_tx_nolock(conn_handle, txom);
+
     ble_hs_unlock();
 
     for (i = 0; i < cid_cnt; i++) {
@@ -845,12 +876,11 @@ ble_l2cap_sig_credit_base_reconfig_req_rx(uint16_t conn_handle,
         ble_l2cap_event_coc_reconfigured(conn_handle, 0, chan[i], true);
     }
 
-    ble_l2cap_sig_tx(conn_handle, txom);
     return 0;
 
 failed:
+    ble_l2cap_sig_tx_nolock(conn_handle, txom);
     ble_hs_unlock();
-    ble_l2cap_sig_tx(conn_handle, txom);
     return 0;
 }
 
@@ -899,7 +929,7 @@ ble_l2cap_sig_credit_base_reconfig_rsp_rx(uint16_t conn_handle,
         return 0;
     }
 
-    rc = ble_hs_mbuf_pullup_base(om, hdr->length);
+    rc = ble_l2cap_sig_mbuf_pullup_base(om, sizeof(*rsp), NULL);
     if (rc != 0) {
         return rc;
     }
@@ -923,17 +953,16 @@ ble_l2cap_sig_credit_base_con_req_rx(uint16_t conn_handle,
     struct ble_l2cap_chan *chans[5] = { 0 };
     struct ble_hs_conn *conn;
     uint16_t scid;
-    uint8_t num_of_scids;
-    uint8_t chan_created = 0;
-    int i;
-    uint8_t len;
+    unsigned int num_of_scids;
+    unsigned int len;
+    unsigned int i;
 
-    rc = ble_hs_mbuf_pullup_base(om, hdr->length);
+    rc = ble_l2cap_sig_mbuf_pullup_base(om, sizeof(*req), &num_of_scids);
     if (rc != 0) {
         return rc;
     }
 
-    len = (hdr->length > sizeof(*req)) ? hdr->length : sizeof(*req);
+    len = sizeof(*rsp) + (num_of_scids * sizeof(rsp->dcids[0]));
 
     rsp = ble_l2cap_sig_cmd_get(BLE_L2CAP_SIG_OP_CREDIT_CONNECT_RSP,
                                 hdr->identifier, len , &txom);
@@ -944,35 +973,20 @@ ble_l2cap_sig_credit_base_con_req_rx(uint16_t conn_handle,
         return 0;
     }
 
-    ble_hs_lock();
-
     memset(rsp, 0, len);
 
-    /* Initial dummy values in case of error, just to satisfy PTS */
-    rsp->credits = htole16(1);
-    rsp->mps = htole16(BLE_L2CAP_ECOC_MIN_MTU);
-    rsp->mtu = htole16(BLE_L2CAP_ECOC_MIN_MTU);
-
-    if (hdr->length <= sizeof(*req)) {
+    if (hdr->length <= sizeof(*req) || num_of_scids > ARRAY_SIZE(chans)) {
         rsp->result = htole16(BLE_L2CAP_COC_ERR_INVALID_PARAMETERS);
         goto failed;
     }
 
     req = (struct ble_l2cap_sig_credit_base_connect_req *)(*om)->om_data;
 
-    num_of_scids = (hdr->length - sizeof(*req)) / sizeof(uint16_t);
-    if (num_of_scids > 5) {
-        rsp->result = htole16(BLE_L2CAP_COC_ERR_INVALID_PARAMETERS);
-        goto failed;
-    }
-
     if ((req->mtu < BLE_L2CAP_ECOC_MIN_MTU) || (req->mps < BLE_L2CAP_ECOC_MIN_MTU)) {
         rsp->result = htole16(BLE_L2CAP_COC_ERR_INVALID_PARAMETERS);
         goto failed;
     }
 
-    conn = ble_hs_conn_find_assert(conn_handle);
-
     /* First verify that provided SCIDs are good */
     for (i = 0; i < num_of_scids; i++) {
         scid = le16toh(req->scids[i]);
@@ -982,30 +996,40 @@ ble_l2cap_sig_credit_base_con_req_rx(uint16_t conn_handle,
         }
     }
 
-    /* Let us try to connect channels */
+    ble_hs_lock();
+    /* we should not handle any disconnects in between unlock-lock so this can
+     * be asserted
+     */
+    conn = ble_hs_conn_find_assert(conn_handle);
+
+    /* Note: to simplify implementation stop processing further channels on
+     * error
+     */
+
     for (i = 0; i < num_of_scids; i++) {
-        /* Verify CID. Note, scid in the request is dcid for out local channel */
         scid = le16toh(req->scids[i]);
-        chans[i] = ble_hs_conn_chan_find_by_dcid(conn, scid);
-        if (chans[i]) {
+
+        /* verify for already used CIDs */
+        if (ble_hs_conn_chan_find_by_dcid(conn, scid)) {
             rsp->result = htole16(BLE_L2CAP_COC_ERR_SOURCE_CID_ALREADY_USED);
-            rsp->dcids[i] = htole16(chans[i]->scid);
             continue;
         }
 
         rc = ble_l2cap_coc_create_srv_chan(conn, le16toh(req->psm), &chans[i]);
         if (rc != 0) {
-            if (i == 0) {
-                /* In case it is very first channel we cannot create it means PSM is incorrect
-                 * or we are out of resources. Just send a response now.
-                 */
+            if (rc == BLE_HS_ENOTSUP) {
+                /* PSM not supported */
                 rsp->result = htole16(ble_l2cap_sig_ble_hs_err2coc_err(rc));
+                ble_hs_unlock();
                 goto failed;
-            } else {
-                /* We cannot create number of channels req by peer due to limited resources. */
-                rsp->result = htole16(BLE_L2CAP_COC_ERR_NO_RESOURCES);
-                goto done;
             }
+
+            /* We cannot process more due to limited resources
+             * If we limit further due to eg. authorization result will be
+             * overwritten later on
+             */
+            rsp->result = htole16(BLE_L2CAP_COC_ERR_NO_RESOURCES);
+            break;
         }
 
         /* Fill up remote configuration. Note MPS is the L2CAP MTU*/
@@ -1015,64 +1039,78 @@ ble_l2cap_sig_credit_base_con_req_rx(uint16_t conn_handle,
         chans[i]->coc_tx.mtu = le16toh(req->mtu);
 
         ble_hs_conn_chan_insert(conn, chans[i]);
-        /* Sending event to the app. Unlock hs */
-        ble_hs_unlock();
 
-        rc = ble_l2cap_event_coc_accept(chans[i], le16toh(req->mtu));
-        if (rc == 0) {
-            rsp->dcids[i] = htole16(chans[i]->scid);
-            chan_created++;
-            if (chan_created == 1) {
-                /* We need to set it once as there are same initial parameters
-                 * for all the channels
-                 */
-                rsp->credits = htole16(chans[i]->coc_rx.credits);
-                rsp->mps = htole16(chans[i]->my_mtu);
-                rsp->mtu = htole16(chans[i]->coc_rx.mtu);
+        /* We need to set it once as there are same initial parameters for all
+         * the channels, regardless if were accepted or not
+         */
+        if (rsp->mtu == 0) {
+            rsp->mtu = htole16(chans[i]->coc_rx.mtu);
+            rsp->mps = htole16(chans[i]->my_mtu);
+            rsp->credits = htole16(chans[i]->coc_rx.credits);
+        }
+    }
+
+    /* further process only channels we have resources for */
+    num_of_scids = i;
+
+    /* Sending events to the app. Unlock hs */
+    ble_hs_unlock();
+
+    for (i = 0; i < num_of_scids; i++) {
+        if (chans[i]) {
+            rc = ble_l2cap_event_coc_accept(chans[i], le16toh(req->mtu));
+            if (rc != 0) {
+                chans[i]->cb = NULL;
+                rsp->result = htole16(ble_l2cap_sig_ble_hs_err2coc_err(rc));
+                continue;
             }
-        } else {
-            /* Make sure we do not send disconnect event when removing channel */
-            chans[i]->cb = NULL;
 
-            ble_hs_lock();
-            conn = ble_hs_conn_find_assert(conn_handle);
+            rsp->dcids[i] = htole16(chans[i]->scid);
+        }
+    }
+
+    ble_hs_lock();
+    conn = ble_hs_conn_find_assert(conn_handle);
+
+    /* removed not accepted channels */
+    for (i = 0; i < num_of_scids; i++) {
+        if (chans[i] && chans[i]->cb == NULL) {
             ble_hs_conn_delete_chan(conn, chans[i]);
             chans[i] = NULL;
-            rsp->result = htole16(ble_l2cap_sig_ble_hs_err2coc_err(rc));
-            rc = 0;
-            ble_hs_unlock();
         }
-
-        ble_hs_lock();
-        conn = ble_hs_conn_find_assert(conn_handle);
     }
 
-done:
-    ble_hs_unlock();
-    rc = ble_l2cap_sig_tx(conn_handle, txom);
+    /* send response */
+    rc = ble_l2cap_sig_tx_nolock(conn_handle, txom);
     if (rc != 0) {
-        ble_hs_lock();
-        conn = ble_hs_conn_find_assert(conn_handle);
+        /* cleanup if we failed to send response
+         * TODO should we notify app on failure?
+         */
         for (i = 0; i < num_of_scids; i++) {
             if (chans[i]) {
+                chans[i]->cb = NULL;
                 ble_hs_conn_delete_chan(conn, chans[i]);
+                chans[i] = NULL;
             }
         }
-        ble_hs_unlock();
-        return 0;
     }
 
+    ble_hs_unlock();
+
     /* Notify user about connection status */
     for (i = 0; i < num_of_scids; i++) {
         if (chans[i]) {
-            ble_l2cap_event_coc_connected(chans[i], rc);
+            ble_l2cap_event_coc_connected(chans[i], 0);
         }
     }
 
     return 0;
 
 failed:
-    ble_hs_unlock();
+    /* dummy values in case of error, just to satisfy PTS */
+    rsp->credits = htole16(1);
+    rsp->mps = htole16(BLE_L2CAP_ECOC_MIN_MTU);
+    rsp->mtu = htole16(BLE_L2CAP_ECOC_MIN_MTU);
     ble_l2cap_sig_tx(conn_handle, txom);
     return 0;
 }
@@ -1089,6 +1127,7 @@ ble_l2cap_sig_credit_base_con_rsp_rx(uint16_t conn_handle,
     int rc;
     int i;
     uint16_t duplicated_cids[5] = {};
+    unsigned int num_of_dcids;
 
 #if !BLE_MONITOR
     BLE_HS_LOG(DEBUG, "L2CAP LE COC connection response received\n");
@@ -1101,11 +1140,16 @@ ble_l2cap_sig_credit_base_con_rsp_rx(uint16_t conn_handle,
         return 0;
     }
 
-    rc = ble_hs_mbuf_pullup_base(om, hdr->length);
+    rc = ble_l2cap_sig_mbuf_pullup_base(om, sizeof(*rsp), &num_of_dcids);
     if (rc != 0) {
         goto done;
     }
 
+    /* spec doesn't say what to do in that case so just fail */
+    if (proc->connect.chan_cnt != num_of_dcids) {
+        goto done;
+    }
+
     rsp = (struct ble_l2cap_sig_credit_base_connect_rsp *)(*om)->om_data;
 
     if (rsp->result) {
@@ -1148,19 +1192,16 @@ ble_l2cap_sig_credit_base_con_rsp_rx(uint16_t conn_handle,
         ble_hs_conn_chan_insert(conn, chan);
     }
 
-    ble_hs_unlock();
-
-done:
-    for (i = 0; i < 5; i++){
-        if (duplicated_cids[i] != 0){
-            ble_hs_lock();
-            conn = ble_hs_conn_find(conn_handle);
+    for (i = 0; i < ARRAY_SIZE(duplicated_cids); i++) {
+        if (duplicated_cids[i] != 0) {
             chan = ble_hs_conn_chan_find_by_dcid(conn, duplicated_cids[i]);
-            ble_hs_unlock();
-            rc = ble_l2cap_sig_disconnect(chan);
+            rc |= ble_l2cap_sig_disconnect_nolock(chan);
         }
     }
 
+    ble_hs_unlock();
+
+done:
     ble_l2cap_sig_coc_connect_cb(proc, rc);
     ble_l2cap_sig_proc_free(proc);
 
@@ -1181,7 +1222,7 @@ ble_l2cap_sig_coc_req_rx(uint16_t conn_handle, struct ble_l2cap_sig_hdr *hdr,
     struct ble_hs_conn *conn;
     uint16_t scid;
 
-    rc = ble_hs_mbuf_pullup_base(om, sizeof(req));
+    rc = ble_l2cap_sig_mbuf_pullup_base(om, sizeof(*req), NULL);
     if (rc != 0) {
         return rc;
     }
@@ -1295,7 +1336,7 @@ ble_l2cap_sig_coc_rsp_rx(uint16_t conn_handle, struct ble_l2cap_sig_hdr *hdr,
         return 0;
     }
 
-    rc = ble_hs_mbuf_pullup_base(om, sizeof(*rsp));
+    rc = ble_l2cap_sig_mbuf_pullup_base(om, sizeof(*rsp), NULL);
     if (rc != 0) {
         goto done;
     }
@@ -1334,9 +1375,9 @@ done:
 }
 
 int
-ble_l2cap_sig_coc_connect(uint16_t conn_handle, uint16_t psm, uint16_t mtu,
-                          struct os_mbuf *sdu_rx,
-                          ble_l2cap_event_fn *cb, void *cb_arg)
+ble_l2cap_sig_connect_nolock(uint16_t conn_handle, uint16_t psm, uint16_t mtu,
+                             struct os_mbuf *sdu_rx, ble_l2cap_event_fn *cb,
+                             void *cb_arg)
 {
     struct ble_hs_conn *conn;
     struct ble_l2cap_sig_proc *proc;
@@ -1349,24 +1390,19 @@ ble_l2cap_sig_coc_connect(uint16_t conn_handle, uint16_t psm, uint16_t mtu,
         return BLE_HS_EINVAL;
     }
 
-    ble_hs_lock();
     conn = ble_hs_conn_find(conn_handle);
-
     if (!conn) {
-        ble_hs_unlock();
         return BLE_HS_ENOTCONN;
     }
 
     chan = ble_l2cap_coc_chan_alloc(conn, psm, mtu, sdu_rx, cb, cb_arg);
     if (!chan) {
-        ble_hs_unlock();
         return BLE_HS_ENOMEM;
     }
 
     proc = ble_l2cap_sig_proc_alloc();
     if (!proc) {
         ble_l2cap_chan_free(conn, chan);
-        ble_hs_unlock();
         return BLE_HS_ENOMEM;
     }
 
@@ -1380,10 +1416,8 @@ ble_l2cap_sig_coc_connect(uint16_t conn_handle, uint16_t psm, uint16_t mtu,
                                 sizeof(*req), &txom);
     if (!req) {
         ble_l2cap_chan_free(conn, chan);
-        ble_hs_unlock();
-        rc = BLE_HS_ENOMEM;
-        /* Goto done to clear proc */
-        goto done;
+        ble_l2cap_sig_proc_free(proc);
+        return BLE_HS_ENOMEM;
     }
 
     req->psm = htole16(psm);
@@ -1392,52 +1426,42 @@ ble_l2cap_sig_coc_connect(uint16_t conn_handle, uint16_t psm, uint16_t mtu,
     req->mps = htole16(chan->my_coc_mps);
     req->credits = htole16(chan->coc_rx.credits);
 
-    ble_hs_unlock();
-
-    rc = ble_l2cap_sig_tx(proc->conn_handle, txom);
+    rc = ble_l2cap_sig_tx_nolock(proc->conn_handle, txom);
     if (rc != 0) {
-        ble_hs_lock();
-        conn = ble_hs_conn_find_assert(conn_handle);
         ble_l2cap_chan_free(conn, chan);
-        ble_hs_unlock();
+        ble_l2cap_sig_proc_free(proc);
+        return rc;
     }
 
-done:
-    ble_l2cap_sig_process_status(proc, rc);
+    ble_l2cap_sig_proc_start(proc);
 
-    return rc;
+    return 0;
 }
 
 #if MYNEWT_VAL(BLE_L2CAP_ENHANCED_COC)
 int
-ble_l2cap_sig_ecoc_connect(uint16_t conn_handle, uint16_t psm, uint16_t mtu,
-                           uint8_t num, struct os_mbuf *sdu_rx[],
-                           ble_l2cap_event_fn *cb, void *cb_arg)
+ble_l2cap_sig_ecoc_connect_nolock(uint16_t conn_handle, uint16_t psm, uint16_t mtu,
+                                  uint8_t num, struct os_mbuf *sdu_rx[],
+                                  ble_l2cap_event_fn *cb, void *cb_arg)
 {
     struct ble_hs_conn *conn;
     struct ble_l2cap_sig_proc *proc;
-    struct ble_l2cap_chan *chan = NULL;
     struct os_mbuf *txom;
     struct ble_l2cap_sig_credit_base_connect_req *req;
     int rc;
     int i;
-    int j;
 
     if (!sdu_rx || !cb) {
         return BLE_HS_EINVAL;
     }
 
-    ble_hs_lock();
     conn = ble_hs_conn_find(conn_handle);
-
     if (!conn) {
-        ble_hs_unlock();
         return BLE_HS_ENOTCONN;
     }
 
     proc = ble_l2cap_sig_proc_alloc();
     if (!proc) {
-        ble_hs_unlock();
         return BLE_HS_ENOMEM;
     }
 
@@ -1448,52 +1472,56 @@ ble_l2cap_sig_ecoc_connect(uint16_t conn_handle, uint16_t psm, uint16_t mtu,
     req = ble_l2cap_sig_cmd_get(BLE_L2CAP_SIG_OP_CREDIT_CONNECT_REQ, proc->id,
                                 sizeof(*req) + num * sizeof(uint16_t), &txom);
     if (!req) {
-        ble_hs_unlock();
-        rc = BLE_HS_ENOMEM;
-        /* Goto done to clear proc */
-        goto done;
+        ble_l2cap_sig_proc_free(proc);
+        return BLE_HS_ENOMEM;
     }
 
     for (i = 0; i < num; i++) {
-        chan = ble_l2cap_coc_chan_alloc(conn, psm, mtu, sdu_rx[i], cb, cb_arg);
-        if (!chan) {
+        proc->connect.chan[i] =
+            ble_l2cap_coc_chan_alloc(conn, psm, mtu, sdu_rx[i], cb, cb_arg);
+        if (!proc->connect.chan[i]) {
             /* Clear request buffer */
             os_mbuf_free_chain(txom);
-
-            for (j = 0; j < i; j++) {
-                /* Clear callback to make sure "Disconnected event" to the user */
-                chan[j].cb = NULL;
-                ble_l2cap_chan_free(conn, proc->connect.chan[j]);
-            }
-            ble_hs_unlock();
             rc = BLE_HS_ENOMEM;
-            goto done;
+            goto failed;
         }
-        proc->connect.chan[i] = chan;
     }
     proc->connect.chan_cnt = num;
 
+    /* all channels have same params */
     req->psm = htole16(psm);
-    req->mtu = htole16(chan->coc_rx.mtu);
-    req->mps = htole16(chan->my_mtu);
-    req->credits = htole16(chan->coc_rx.credits);
+    req->mtu = htole16(proc->connect.chan[0]->coc_rx.mtu);
+    req->mps = htole16(proc->connect.chan[0]->my_mtu);
+    req->credits = htole16(proc->connect.chan[0]->coc_rx.credits);
     for (i = 0; i < num; i++) {
         req->scids[i] = htole16(proc->connect.chan[i]->scid);
     }
 
-    ble_hs_unlock();
+    rc = ble_l2cap_sig_tx_nolock(proc->conn_handle, txom);
+    if (rc) {
+        rc = BLE_HS_ENOMEM;
+        goto failed;
+    }
 
-    rc = ble_l2cap_sig_tx(proc->conn_handle, txom);
+    ble_l2cap_sig_proc_start(proc);
 
-done:
-    ble_l2cap_sig_process_status(proc, rc);
+    return 0;
 
-    return rc;
+failed:
+    /* clean up on failure, ble_l2cap_chan_free() handles NULL as well */
+    for (i = 0; i < num; i++) {
+        proc->connect.chan[i]->cb = NULL;
+        ble_l2cap_chan_free(conn, proc->connect.chan[i]);
+    }
+
+    ble_l2cap_sig_proc_free(proc);
+    return BLE_HS_ENOMEM;
 }
 
 int
-ble_l2cap_sig_coc_reconfig(uint16_t conn_handle, struct ble_l2cap_chan *chans[],
-                           uint8_t num, uint16_t new_mtu)
+ble_l2cap_sig_coc_reconfig_nolock(uint16_t conn_handle,
+                                  struct ble_l2cap_chan *chans[], uint8_t num,
+                                  uint16_t new_mtu)
 {
     struct ble_hs_conn *conn;
     struct ble_l2cap_sig_proc *proc;
@@ -1502,17 +1530,13 @@ ble_l2cap_sig_coc_reconfig(uint16_t conn_handle, struct ble_l2cap_chan *chans[],
     int rc;
     int i;
 
-    ble_hs_lock();
     conn = ble_hs_conn_find(conn_handle);
-
     if (!conn) {
-        ble_hs_unlock();
         return BLE_HS_ENOTCONN;
     }
 
     proc = ble_l2cap_sig_proc_alloc();
     if (!proc) {
-        ble_hs_unlock();
         return BLE_HS_ENOMEM;
     }
 
@@ -1520,9 +1544,8 @@ ble_l2cap_sig_coc_reconfig(uint16_t conn_handle, struct ble_l2cap_chan *chans[],
         if (ble_hs_conn_chan_exist(conn, chans[i])) {
             proc->reconfig.cids[i] = chans[i]->scid;
         } else {
-            ble_hs_unlock();
-            rc = BLE_HS_ENOMEM;
-            goto done;
+            ble_l2cap_sig_proc_free(proc);
+            return BLE_HS_ENOMEM;
         }
     }
 
@@ -1536,9 +1559,8 @@ ble_l2cap_sig_coc_reconfig(uint16_t conn_handle, struct ble_l2cap_chan *chans[],
     req = ble_l2cap_sig_cmd_get(BLE_L2CAP_SIG_OP_CREDIT_RECONFIG_REQ, proc->id,
                                 sizeof(*req) + num * sizeof(uint16_t), &txom);
     if (!req) {
-        ble_hs_unlock();
-        rc = BLE_HS_ENOMEM;
-        goto done;
+        ble_l2cap_sig_proc_free(proc);
+        return BLE_HS_ENOMEM;
     }
 
     /* For now we allow to change CoC MTU only.*/
@@ -1549,14 +1571,14 @@ ble_l2cap_sig_coc_reconfig(uint16_t conn_handle, struct ble_l2cap_chan *chans[],
         req->dcids[i] = htole16(proc->reconfig.cids[i]);
     }
 
-    ble_hs_unlock();
+    rc = ble_l2cap_sig_tx_nolock(proc->conn_handle, txom);
+    if (rc) {
+        ble_l2cap_sig_proc_free(proc);
+        return rc;
+    }
 
-    rc = ble_l2cap_sig_tx(proc->conn_handle, txom);
-
-done:
-    ble_l2cap_sig_process_status(proc, rc);
-
-    return rc;
+    ble_l2cap_sig_proc_start(proc);
+    return 0;
 }
 #endif
 
@@ -1574,8 +1596,10 @@ ble_l2cap_sig_disc_req_rx(uint16_t conn_handle, struct ble_l2cap_sig_hdr *hdr,
     struct ble_l2cap_chan *chan;
     struct ble_hs_conn *conn;
     int rc;
+    uint16_t scid;
+    uint16_t dcid;
 
-    rc = ble_hs_mbuf_pullup_base(om, sizeof(*req));
+    rc = ble_l2cap_sig_mbuf_pullup_base(om, sizeof(*req), NULL);
     if (rc != 0) {
         return rc;
     }
@@ -1593,18 +1617,34 @@ ble_l2cap_sig_disc_req_rx(uint16_t conn_handle, struct ble_l2cap_sig_hdr *hdr,
     conn = ble_hs_conn_find_assert(conn_handle);
 
     req = (struct ble_l2cap_sig_disc_req *) (*om)->om_data;
+    scid = le16toh(req->scid);
+    dcid = le16toh(req->dcid);
+
+    if (scid < BLE_L2CAP_COC_CID_START || scid > BLE_L2CAP_COC_CID_END ||
+        dcid < BLE_L2CAP_COC_CID_START || dcid > BLE_L2CAP_COC_CID_END) {
+        /* Don't bother with look-up if it is not for connection oriented
+         * channel
+         */
+        chan = NULL;
+    } else {
+        /* Let's find matching channel. Note that destination CID in the request
+         * is from peer perspective. It is source CID from nimble perspective
+         */
+        chan = ble_hs_conn_chan_find_by_scid(conn, dcid);
+    }
 
-    /* Let's find matching channel. Note that destination CID in the request
-     * is from peer perspective. It is source CID from nimble perspective
-     */
-    chan = ble_hs_conn_chan_find_by_scid(conn, le16toh(req->dcid));
     if (!chan) {
         os_mbuf_free_chain(txom);
         ble_hs_unlock();
         ble_l2cap_sig_reject_invalid_cid_tx(conn_handle, hdr->identifier, req->dcid, req->scid);
         return 0;
     }
-    if (le16toh(req->scid) != chan->dcid) {
+
+    /* Core Spec 6.0 Vol3 PartA 4.6
+     * If the receiver finds a DCID match but the SCID fails to find the same
+     * match, the request should be silently discarded.
+     */
+    if (scid != chan->dcid) {
         os_mbuf_free_chain(txom);
         ble_hs_unlock();
         return 0;
@@ -1627,25 +1667,15 @@ static void
 ble_l2cap_sig_coc_disconnect_cb(struct ble_l2cap_sig_proc *proc, int status)
 {
     struct ble_l2cap_chan *chan;
-    struct ble_l2cap_event event;
     struct ble_hs_conn *conn;
 
-    if (!proc) {
-        return;
-    }
+    assert(proc);
 
-    memset(&event, 0, sizeof(event));
     chan = proc->disconnect.chan;
-
     if (!chan) {
         return;
     }
 
-    if (!chan->cb) {
-        goto done;
-    }
-
-done:
     ble_hs_lock();
     conn = ble_hs_conn_find_assert(chan->conn_handle);
     if (conn) {
@@ -1672,7 +1702,7 @@ ble_l2cap_sig_disc_rsp_rx(uint16_t conn_handle, struct ble_l2cap_sig_hdr *hdr,
         return 0;
     }
 
-    rc = ble_hs_mbuf_pullup_base(om, sizeof(*rsp));
+    rc = ble_l2cap_sig_mbuf_pullup_base(om, sizeof(*rsp), NULL);
     if (rc != 0) {
         goto done;
     }
@@ -1685,7 +1715,7 @@ ble_l2cap_sig_disc_rsp_rx(uint16_t conn_handle, struct ble_l2cap_sig_hdr *hdr,
     rsp = (struct ble_l2cap_sig_disc_rsp *)(*om)->om_data;
     if (chan->dcid != le16toh(rsp->dcid) || chan->scid != le16toh(rsp->scid)) {
         /* This response is incorrect, lets wait for timeout */
-        ble_l2cap_sig_process_status(proc, 0);
+        ble_l2cap_sig_proc_start(proc);
         return 0;
     }
 
@@ -1697,13 +1727,18 @@ done:
 }
 
 int
-ble_l2cap_sig_disconnect(struct ble_l2cap_chan *chan)
+ble_l2cap_sig_disconnect_nolock(struct ble_l2cap_chan *chan)
 {
     struct os_mbuf *txom;
     struct ble_l2cap_sig_disc_req *req;
     struct ble_l2cap_sig_proc *proc;
     int rc;
 
+    /* this is allowed only for connection oriented channels */
+    if (chan->scid < BLE_L2CAP_COC_CID_START || chan->scid > BLE_L2CAP_COC_CID_END) {
+        return BLE_HS_EREJECT;
+    }
+
     if (chan->flags & BLE_L2CAP_CHAN_F_DISCONNECTING) {
         return 0;
     }
@@ -1721,23 +1756,24 @@ ble_l2cap_sig_disconnect(struct ble_l2cap_chan *chan)
     req = ble_l2cap_sig_cmd_get(BLE_L2CAP_SIG_OP_DISCONN_REQ, proc->id,
                                 sizeof(*req), &txom);
     if (!req) {
-        rc = BLE_HS_ENOMEM;
-        goto done;
+        ble_l2cap_sig_proc_free(proc);
+        return BLE_HS_ENOMEM;
     }
 
     req->dcid = htole16(chan->dcid);
     req->scid = htole16(chan->scid);
 
-    rc = ble_l2cap_sig_tx(proc->conn_handle, txom);
-    /* Mark channel as disconnecting */
-    if (rc == 0) {
-        chan->flags |= BLE_L2CAP_CHAN_F_DISCONNECTING;
+    rc = ble_l2cap_sig_tx_nolock(proc->conn_handle, txom);
+    if (rc) {
+        ble_l2cap_sig_proc_free(proc);
+        return rc;
     }
 
-done:
-    ble_l2cap_sig_process_status(proc, rc);
+    /* Mark channel as disconnecting */
+    chan->flags |= BLE_L2CAP_CHAN_F_DISCONNECTING;
+    ble_l2cap_sig_proc_start(proc);
 
-    return rc;
+    return 0;
 }
 
 static int
@@ -1747,7 +1783,7 @@ ble_l2cap_sig_le_credits_rx(uint16_t conn_handle, struct ble_l2cap_sig_hdr *hdr,
     struct ble_l2cap_sig_le_credits *req;
     int rc;
 
-    rc = ble_hs_mbuf_pullup_base(om, sizeof(*req));
+    rc = ble_l2cap_sig_mbuf_pullup_base(om, sizeof(*req), NULL);
     if (rc != 0) {
         return 0;
     }
@@ -1766,7 +1802,7 @@ ble_l2cap_sig_le_credits_rx(uint16_t conn_handle, struct ble_l2cap_sig_hdr *hdr,
 }
 
 int
-ble_l2cap_sig_le_credits(uint16_t conn_handle, uint16_t scid, uint16_t credits)
+ble_l2cap_sig_le_credits_nolock(uint16_t conn_handle, uint16_t scid, uint16_t credits)
 {
     struct ble_l2cap_sig_le_credits *cmd;
     struct os_mbuf *txom;
@@ -1781,7 +1817,19 @@ ble_l2cap_sig_le_credits(uint16_t conn_handle, uint16_t scid, uint16_t credits)
     cmd->scid = htole16(scid);
     cmd->credits = htole16(credits);
 
-    return ble_l2cap_sig_tx(conn_handle, txom);
+    return ble_l2cap_sig_tx_nolock(conn_handle, txom);
+}
+
+int
+ble_l2cap_sig_le_credits(uint16_t conn_handle, uint16_t scid, uint16_t credits)
+{
+    int rc;
+
+    ble_hs_lock();
+    rc = ble_l2cap_sig_le_credits_nolock(conn_handle, scid, credits);
+    ble_hs_unlock();
+
+    return rc;
 }
 #endif
 
@@ -1798,14 +1846,14 @@ ble_l2cap_sig_rx_reject(uint16_t conn_handle,
        return 0;
    }
 
-   switch (proc->id) {
+   switch (proc->op) {
 #if MYNEWT_VAL(BLE_L2CAP_COC_MAX_NUM) != 0
-       case BLE_L2CAP_SIG_PROC_OP_CONNECT:
-           ble_l2cap_sig_coc_connect_cb(proc, BLE_HS_EREJECT);
-           break;
+   case BLE_L2CAP_SIG_PROC_OP_CONNECT:
+       ble_l2cap_sig_coc_connect_cb(proc, BLE_HS_EREJECT);
+       break;
 #endif
-       default:
-           break;
+   default:
+       break;
    }
 
    ble_l2cap_sig_proc_free(proc);
@@ -1816,16 +1864,14 @@ ble_l2cap_sig_rx_reject(uint16_t conn_handle,
  *****************************************************************************/
 
 static int
-ble_l2cap_sig_rx(struct ble_l2cap_chan *chan)
+ble_l2cap_sig_rx(struct ble_l2cap_chan *chan, struct os_mbuf **om)
 {
     struct ble_l2cap_sig_hdr hdr;
     ble_l2cap_sig_rx_fn *rx_cb;
     uint16_t conn_handle;
-    struct os_mbuf **om;
     int rc;
 
     conn_handle = chan->conn_handle;
-    om = &chan->rx_buf;
 
     STATS_INC(ble_l2cap_stats, sig_rx);
 
@@ -1939,31 +1985,37 @@ ble_l2cap_sig_conn_broken(uint16_t conn_handle, int reason)
 {
     struct ble_l2cap_sig_proc *proc;
 
-    /* Report a failure for each timed out procedure. */
-    while ((proc = STAILQ_FIRST(&ble_l2cap_sig_procs)) != NULL) {
-        switch(proc->op) {
-            case BLE_L2CAP_SIG_PROC_OP_UPDATE:
-                ble_l2cap_sig_update_call_cb(proc, reason);
-                break;
-#if MYNEWT_VAL(BLE_L2CAP_COC_MAX_NUM) != 0
-            case BLE_L2CAP_SIG_PROC_OP_CONNECT:
-                ble_l2cap_sig_coc_connect_cb(proc, reason);
-            break;
-            case BLE_L2CAP_SIG_PROC_OP_DISCONNECT:
-                ble_l2cap_sig_coc_disconnect_cb(proc, reason);
-            break;
-#if MYNEWT_VAL(BLE_L2CAP_ENHANCED_COC)
-            case BLE_L2CAP_SIG_PROC_OP_RECONFIG:
-                ble_l2cap_sig_coc_reconfig_cb(proc, reason);
-            break;
-#endif
-#endif
-            }
-
-            STAILQ_REMOVE_HEAD(&ble_l2cap_sig_procs, next);
-            ble_l2cap_sig_proc_free(proc);
+    /* If there were any pending procedure, indicate to the application that it
+     * did not complete.
+     */
+    proc = ble_l2cap_sig_proc_extract(conn_handle, BLE_L2CAP_SIG_PROC_OP_UPDATE, 0);
+    if (proc != NULL) {
+        ble_l2cap_sig_update_call_cb(proc, reason);
+        ble_l2cap_sig_proc_free(proc);
     }
 
+#if MYNEWT_VAL(BLE_L2CAP_COC_MAX_NUM) != 0
+    proc = ble_l2cap_sig_proc_extract(conn_handle, BLE_L2CAP_SIG_PROC_OP_CONNECT, 0);
+    if (proc != NULL) {
+        ble_l2cap_sig_coc_connect_cb(proc, reason);
+        ble_l2cap_sig_proc_free(proc);
+    }
+
+    proc = ble_l2cap_sig_proc_extract(conn_handle,
+                                      BLE_L2CAP_SIG_PROC_OP_DISCONNECT, 0);
+    if (proc != NULL) {
+        ble_l2cap_sig_coc_disconnect_cb(proc, reason);
+        ble_l2cap_sig_proc_free(proc);
+    }
+
+#if MYNEWT_VAL(BLE_L2CAP_ENHANCED_COC)
+    proc = ble_l2cap_sig_proc_extract(conn_handle, BLE_L2CAP_SIG_PROC_OP_RECONFIG, 0);
+    if (proc != NULL) {
+        ble_l2cap_sig_coc_reconfig_cb(proc, reason);
+        ble_l2cap_sig_proc_free(proc);
+    }
+#endif
+#endif
 }
 
 /**
@@ -1989,15 +2041,15 @@ ble_l2cap_sig_timer(void)
     while ((proc = STAILQ_FIRST(&temp_list)) != NULL) {
         STATS_INC(ble_l2cap_stats, proc_timeout);
         switch(proc->op) {
-            case BLE_L2CAP_SIG_PROC_OP_UPDATE:
-                ble_l2cap_sig_update_call_cb(proc, BLE_HS_ETIMEOUT);
-                break;
-#if MYNEWT_VAL(BLE_L2CAP_COC_MAX_NUM) != 0
-            case BLE_L2CAP_SIG_PROC_OP_CONNECT:
-                ble_l2cap_sig_coc_connect_cb(proc, BLE_HS_ETIMEOUT);
+        case BLE_L2CAP_SIG_PROC_OP_UPDATE:
+            ble_l2cap_sig_update_call_cb(proc, BLE_HS_ETIMEOUT);
             break;
-            case BLE_L2CAP_SIG_PROC_OP_DISCONNECT:
-                ble_l2cap_sig_coc_disconnect_cb(proc, BLE_HS_ETIMEOUT);
+#if MYNEWT_VAL(BLE_L2CAP_COC_MAX_NUM) != 0
+        case BLE_L2CAP_SIG_PROC_OP_CONNECT:
+            ble_l2cap_sig_coc_connect_cb(proc, BLE_HS_ETIMEOUT);
+            break;
+        case BLE_L2CAP_SIG_PROC_OP_DISCONNECT:
+            ble_l2cap_sig_coc_disconnect_cb(proc, BLE_HS_ETIMEOUT);
             break;
 #endif
         }
