@@ -23,7 +23,7 @@
 #include "sysinit/sysinit.h"
 #include "syscfg/syscfg.h"
 #include "host/ble_hs.h"
-#include "base64/base64.h"
+#include "os/util.h"
 #include "store/config/ble_store_config.h"
 #include "ble_store_config_priv.h"
 
