@@ -124,7 +124,7 @@ ble_l2cap_test_util_verify_tx_update_conn(
 }
 
 static int
-ble_l2cap_test_util_dummy_rx(struct ble_l2cap_chan *chan)
+ble_l2cap_test_util_dummy_rx(struct ble_l2cap_chan *chan, struct os_mbuf **om)
 {
     return 0;
 }
@@ -220,8 +220,7 @@ ble_l2cap_test_util_verify_first_frag(uint16_t conn_handle,
 
     conn = ble_hs_conn_find(conn_handle);
     TEST_ASSERT_FATAL(conn != NULL);
-    TEST_ASSERT(conn->bhc_rx_chan != NULL &&
-                conn->bhc_rx_chan->scid == BLE_L2CAP_TEST_CID);
+    TEST_ASSERT(conn->rx_len && conn->rx_cid == BLE_L2CAP_TEST_CID);
 
     ble_hs_unlock();
 }
@@ -240,8 +239,7 @@ ble_l2cap_test_util_verify_middle_frag(uint16_t conn_handle,
 
     conn = ble_hs_conn_find(conn_handle);
     TEST_ASSERT_FATAL(conn != NULL);
-    TEST_ASSERT(conn->bhc_rx_chan != NULL &&
-                conn->bhc_rx_chan->scid == BLE_L2CAP_TEST_CID);
+    TEST_ASSERT(conn->rx_len && conn->rx_cid == BLE_L2CAP_TEST_CID);
 
     ble_hs_unlock();
 }
@@ -260,7 +258,7 @@ ble_l2cap_test_util_verify_last_frag(uint16_t conn_handle,
 
     conn = ble_hs_conn_find(conn_handle);
     TEST_ASSERT_FATAL(conn != NULL);
-    TEST_ASSERT(conn->bhc_rx_chan == NULL);
+    TEST_ASSERT(conn->rx_len == 0);
 
     ble_hs_unlock();
 }
@@ -279,7 +277,7 @@ TEST_CASE_SELF(ble_l2cap_test_case_bad_header)
                                     NULL, NULL);
 
     rc = ble_l2cap_test_util_rx_first_frag(2, 14, 1234, 10);
-    TEST_ASSERT(rc == BLE_HS_ENOENT);
+    TEST_ASSERT(rc == BLE_HS_EBADDATA);
 
     ble_hs_test_util_assert_mbufs_freed(NULL);
 }
@@ -386,8 +384,7 @@ TEST_CASE_SELF(ble_l2cap_test_case_frag_channels)
     ble_hs_lock();
     conn = ble_hs_conn_find(2);
     TEST_ASSERT_FATAL(conn != NULL);
-    TEST_ASSERT(conn->bhc_rx_chan != NULL &&
-                conn->bhc_rx_chan->scid == BLE_L2CAP_TEST_CID);
+    TEST_ASSERT(conn->rx_len && conn->rx_cid == BLE_L2CAP_TEST_CID);
     ble_hs_unlock();
 
     /* Receive a starting fragment on a different channel.  The first fragment
@@ -400,8 +397,7 @@ TEST_CASE_SELF(ble_l2cap_test_case_frag_channels)
     ble_hs_lock();
     conn = ble_hs_conn_find(2);
     TEST_ASSERT_FATAL(conn != NULL);
-    TEST_ASSERT(conn->bhc_rx_chan != NULL &&
-                conn->bhc_rx_chan->scid == BLE_L2CAP_CID_ATT);
+    TEST_ASSERT(conn->rx_len && conn->rx_cid == BLE_L2CAP_CID_ATT);
     ble_hs_unlock();
 
     /* Terminate the connection.  The received fragments should get freed.
@@ -415,8 +411,12 @@ TEST_CASE_SELF(ble_l2cap_test_case_frag_channels)
 TEST_CASE_SELF(ble_l2cap_test_case_frag_timeout)
 {
     int32_t ticks_from_now;
+    uint32_t timeout_ticks;
     int rc;
 
+    timeout_ticks =
+        ble_npl_time_ms_to_ticks32(MYNEWT_VAL(BLE_L2CAP_RX_FRAG_TIMEOUT));
+
     ble_l2cap_test_util_init();
 
     ble_l2cap_test_util_create_conn(2, ((uint8_t[]){1,2,3,4,5,6}),
@@ -432,10 +432,10 @@ TEST_CASE_SELF(ble_l2cap_test_case_frag_timeout)
 
     /* Ensure timer will expire in 30 seconds. */
     ticks_from_now = ble_hs_conn_timer();
-    TEST_ASSERT(ticks_from_now == MYNEWT_VAL(BLE_L2CAP_RX_FRAG_TIMEOUT));
+    TEST_ASSERT(ticks_from_now == timeout_ticks);
 
     /* Almost let the timer expire. */
-    os_time_advance(MYNEWT_VAL(BLE_L2CAP_RX_FRAG_TIMEOUT) - 1);
+    os_time_advance(timeout_ticks - 1);
     ticks_from_now = ble_hs_conn_timer();
     TEST_ASSERT(ticks_from_now == 1);
 
@@ -445,11 +445,11 @@ TEST_CASE_SELF(ble_l2cap_test_case_frag_timeout)
 
     /* Ensure timer got reset. */
     ticks_from_now = ble_hs_conn_timer();
-    TEST_ASSERT(ticks_from_now == MYNEWT_VAL(BLE_L2CAP_RX_FRAG_TIMEOUT));
+    TEST_ASSERT(ticks_from_now == timeout_ticks);
 
     /* Allow the timer to expire. */
     ble_hs_test_util_hci_ack_set_disconnect(0);
-    os_time_advance(MYNEWT_VAL(BLE_L2CAP_RX_FRAG_TIMEOUT));
+    os_time_advance(timeout_ticks);
     ticks_from_now = ble_hs_conn_timer();
     TEST_ASSERT(ticks_from_now == BLE_HS_FOREVER);
 
@@ -765,14 +765,16 @@ ble_l2cap_test_coc_connect_multi(struct test_data *t)
 {
     struct ble_l2cap_sig_credit_base_connect_req *req;
     struct ble_l2cap_sig_credit_base_connect_rsp *rsp;
+    uint8_t req_buf[sizeof(*req) + (sizeof(uint16_t) * t->num)];
+    uint8_t rsp_buf[sizeof(*rsp) + (sizeof(uint16_t) * t->num)];
     struct os_mbuf *sdu_rx[t->num];
     struct event *ev = &t->event[t->event_iter++];
     uint8_t id;
     int rc;
     int i;
 
-    req = malloc(sizeof(*req) + (sizeof(uint16_t) * t->num));
-    rsp = malloc(sizeof(*rsp) + (sizeof(uint16_t) * t->num));
+    req = (void *)req_buf;
+    rsp = (void *)rsp_buf;
 
     ble_l2cap_test_util_init();
 
@@ -784,8 +786,8 @@ ble_l2cap_test_coc_connect_multi(struct test_data *t)
         assert(sdu_rx[i] != NULL);
     }
 
-    rc = ble_l2cap_sig_ecoc_connect(2, t->psm, t->mtu, t->num, sdu_rx,
-                                   ble_l2cap_test_event, t);
+    rc = ble_l2cap_enhanced_connect(2, t->psm, t->mtu, t->num, sdu_rx,
+                                    ble_l2cap_test_event, t);
     TEST_ASSERT_FATAL(rc == ev->early_error);
 
     if (rc != 0) {
@@ -849,8 +851,7 @@ ble_l2cap_test_coc_connect(struct test_data *t)
     sdu_rx = os_mbuf_get_pkthdr(&sdu_os_mbuf_pool, 0);
     assert(sdu_rx != NULL);
 
-    rc = ble_l2cap_sig_coc_connect(2, t->psm, t->mtu, sdu_rx,
-                                   ble_l2cap_test_event, t);
+    rc = ble_l2cap_connect(2, t->psm, t->mtu, sdu_rx, ble_l2cap_test_event, t);
     TEST_ASSERT_FATAL(rc == ev->early_error);
 
     if (rc != 0) {
@@ -953,7 +954,7 @@ ble_l2cap_test_coc_disc(struct test_data *t)
     uint8_t id;
     int rc;
 
-    rc = ble_l2cap_sig_disconnect(t->chan[0]);
+    rc = ble_l2cap_disconnect(t->chan[0]);
     TEST_ASSERT_FATAL(rc == 0);
 
     req.dcid = htole16(t->chan[0]->dcid);
@@ -1530,6 +1531,29 @@ TEST_CASE_SELF(ble_l2cap_test_case_sig_coc_conn_multi)
     ble_hs_test_util_assert_mbufs_freed(NULL);
 }
 
+TEST_CASE_SELF(ble_l2cap_test_case_server_create_remove)
+{
+    int rc;
+
+    /* Register server */
+    rc = ble_l2cap_create_server(BLE_L2CAP_TEST_PSM, BLE_L2CAP_TEST_COC_MTU,
+                                 ble_l2cap_test_event, NULL);
+    TEST_ASSERT(rc == 0);
+
+    /* Register second server with same PSM */
+    rc = ble_l2cap_create_server(BLE_L2CAP_TEST_PSM, BLE_L2CAP_TEST_COC_MTU,
+                                 ble_l2cap_test_event, NULL);
+    TEST_ASSERT(rc == BLE_HS_EALREADY);
+
+    /* remove success */
+    rc = ble_l2cap_remove_server(BLE_L2CAP_TEST_PSM);
+    TEST_ASSERT(rc == 0);
+
+    /* second remove shall fail */
+    rc = ble_l2cap_remove_server(BLE_L2CAP_TEST_PSM);
+    TEST_ASSERT(rc == BLE_HS_ENOENT);
+}
+
 TEST_SUITE(ble_l2cap_test_suite)
 {
     ble_l2cap_test_case_bad_header();
@@ -1561,4 +1585,5 @@ TEST_SUITE(ble_l2cap_test_suite)
     ble_l2cap_test_case_coc_send_data_failed_too_big_sdu();
     ble_l2cap_test_case_coc_recv_data_succeed();
     ble_l2cap_test_case_sig_coc_conn_multi();
+    ble_l2cap_test_case_server_create_remove();
 }
