@@ -1396,6 +1396,16 @@ struct ble_hci_vs_set_local_irk_cp {
     uint8_t irk[16];
 } __attribute__((packed));
 
+#define BLE_HCI_VS_SET_SCAN_CFG_FLAG_NO_LEGACY               (0x00000001)
+#define BLE_HCI_VS_SET_SCAN_CFG_FLAG_NO_EXT                  (0x00000002)
+#define BLE_HCI_VS_SET_SCAN_CFG_FLAG_RSSI_FILTER             (0x00000004)
+
+#define BLE_HCI_OCF_VS_SET_SCAN_CFG                     (MYNEWT_VAL(BLE_HCI_VS_OCF_OFFSET) + (0x000B))
+struct ble_hci_vs_set_scan_cfg_cp {
+    uint32_t flags;
+    int8_t rssi_threshold;
+} __attribute__((packed));
+
 /* Command Specific Definitions */
 /* --- Set controller to host flow control (OGF 0x03, OCF 0x0031) --- */
 #define BLE_HCI_CTLR_TO_HOST_FC_OFF         (0)
@@ -1637,6 +1647,11 @@ struct ble_hci_vs_set_local_irk_cp {
 #define BLE_HCI_PRIVACY_NETWORK                     (0)
 #define BLE_HCI_PRIVACY_DEVICE                      (1)
 
+/* --- LE iso transmit test  payload type options (OCF 0x0070) */
+#define BLE_HCI_PAYLOAD_TYPE_ZERO_LENGTH        (0x00)
+#define BLE_HCI_PAYLOAD_TYPE_VARIABLE_LENGTH    (0x01)
+#define BLE_HCI_PAYLOAD_TYPE_MAXIMUM_LENGTH     (0x02)
+
 /* --- LE set advertising coded PHY options (OCF 0x007F) */
 #define BLE_HCI_ADVERTISING_PHY_OPT_NO_PREF      0x0
 #define BLE_HCI_ADVERTISING_PHY_OPT_S2_PREF      0x1
@@ -2363,6 +2378,8 @@ struct ble_hci_ev_le_subev_cs_test_end_complete {
 #define BLE_HCI_VER_BCS_5_2                 (11)
 #define BLE_HCI_VER_BCS_5_3                 (12)
 #define BLE_HCI_VER_BCS_5_4                 (13)
+#define BLE_HCI_VER_BCS_6_0                 (14)
+#define BLE_HCI_VER_BCS_6_1                 (15)
 
 #define BLE_LMP_VER_BCS_1_0b                (0)
 #define BLE_LMP_VER_BCS_1_1                 (1)
@@ -2378,6 +2395,8 @@ struct ble_hci_ev_le_subev_cs_test_end_complete {
 #define BLE_LMP_VER_BCS_5_2                 (11)
 #define BLE_LMP_VER_BCS_5_3                 (12)
 #define BLE_LMP_VER_BCS_5_4                 (13)
+#define BLE_LMP_VER_BCS_6_0                 (14)
+#define BLE_LMP_VER_BCS_6_1                 (15)
 
 /* selected HCI and LMP version */
 #if MYNEWT_VAL(BLE_VERSION) == 50
@@ -2395,6 +2414,14 @@ struct ble_hci_ev_le_subev_cs_test_end_complete {
 #elif MYNEWT_VAL(BLE_VERSION) == 54
 #define BLE_HCI_VER_BCS BLE_HCI_VER_BCS_5_4
 #define BLE_LMP_VER_BCS BLE_LMP_VER_BCS_5_4
+#elif MYNEWT_VAL(BLE_VERSION) == 60
+#define BLE_HCI_VER_BCS BLE_HCI_VER_BCS_6_0
+#define BLE_LMP_VER_BCS BLE_LMP_VER_BCS_6_0
+#elif MYNEWT_VAL(BLE_VERSION) == 61
+#define BLE_HCI_VER_BCS BLE_HCI_VER_BCS_6_1
+#define BLE_LMP_VER_BCS BLE_LMP_VER_BCS_6_1
+#else
+#error Unsupported BLE_VERSION selected
 #endif
 
 #define BLE_HCI_DATA_HDR_SZ                 4
@@ -2416,7 +2443,7 @@ struct hci_data_hdr
 #define BLE_HCI_ISO_CONN_HANDLE_MASK        (0x07ff)
 #define BLE_HCI_ISO_PB_FLAG_MASK            (0x3000)
 #define BLE_HCI_ISO_TS_FLAG_MASK            (0x4000)
-#define BLE_HCI_ISO_LENGTH_MASK             (0x7fff)
+#define BLE_HCI_ISO_LENGTH_MASK             (0x3fff)
 #define BLE_HCI_ISO_SDU_LENGTH_MASK         (0x0fff)
 #define BLE_HCI_ISO_PKT_STATUS_FLAG_MASK    (0xC000)
 
@@ -2449,6 +2476,10 @@ struct hci_data_hdr
 
 #define BLE_HCI_ISO_DATA_PATH_ID_HCI            0x00
 
+#define BLE_HCI_ISO_FRAMING_UNFRAMED            0x00
+#define BLE_HCI_ISO_FRAMING_FRAMED_SEGMENTABLE  0x01
+#define BLE_HCI_ISO_FRAMING_FRAMED_UNSEGMENTED  0x02
+
 struct ble_hci_iso {
     uint16_t handle;
     uint16_t length;
