@@ -369,6 +369,10 @@
 #define MYNEWT_VAL_MODLOG_SYSINIT_STAGE (100)
 #endif
 
+#ifndef MYNEWT_VAL_MODLOG_USE_PRINTF_ATTRIBUTE
+#define MYNEWT_VAL_MODLOG_USE_PRINTF_ATTRIBUTE (0)
+#endif
+
 #ifndef MYNEWT_VAL_LOG_CONSOLE
 #define MYNEWT_VAL_LOG_CONSOLE (1)
 #endif
