@@ -49,15 +49,12 @@
 #define max(a, b) ((a) > (b) ? (a) : (b))
 #endif
 
-/**
- * @addtogroup OSKernel
- * @{
- *   @defgroup OSMqueue Queue of Mbufs
- *   @{
- */
+#include "syscfg/syscfg.h"
+#if !MYNEWT_VAL(OS_SYSVIEW_TRACE_MBUF)
+#define OS_TRACE_DISABLE_FILE_API
+#endif
+
 
-STAILQ_HEAD(, os_mbuf_pool) g_msys_pool_list =
-    STAILQ_HEAD_INITIALIZER(g_msys_pool_list);
 
 
 int
@@ -97,19 +94,19 @@ os_mqueue_get(struct os_mqueue *mq)
 }
 
 int
-os_mqueue_put(struct os_mqueue *mq, struct ble_npl_eventq *evq, struct os_mbuf *m)
+os_mqueue_put(struct os_mqueue *mq, struct ble_npl_eventq *evq, struct os_mbuf *om)
 {
     struct os_mbuf_pkthdr *mp;
     os_sr_t sr;
     int rc;
 
     /* Can only place the head of a chained mbuf on the queue. */
-    if (!OS_MBUF_IS_PKTHDR(m)) {
+    if (!OS_MBUF_IS_PKTHDR(om)) {
         rc = OS_EINVAL;
         goto err;
     }
 
-    mp = OS_MBUF_PKTHDR(m);
+    mp = OS_MBUF_PKTHDR(om);
 
     OS_ENTER_CRITICAL(sr);
     STAILQ_INSERT_TAIL(&mq->mq_head, mp, omp_next);
@@ -125,141 +122,6 @@ err:
     return (rc);
 }
 
-int
-os_msys_register(struct os_mbuf_pool *new_pool)
-{
-    struct os_mbuf_pool *pool;
-
-    pool = NULL;
-    STAILQ_FOREACH(pool, &g_msys_pool_list, omp_next) {
-        if (new_pool->omp_databuf_len > pool->omp_databuf_len) {
-            break;
-        }
-    }
-
-    if (pool) {
-        STAILQ_INSERT_AFTER(&g_msys_pool_list, pool, new_pool, omp_next);
-    } else {
-        STAILQ_INSERT_TAIL(&g_msys_pool_list, new_pool, omp_next);
-    }
-
-    return (0);
-}
-
-void
-os_msys_reset(void)
-{
-    STAILQ_INIT(&g_msys_pool_list);
-}
-
-static struct os_mbuf_pool *
-os_msys_find_pool(uint16_t dsize)
-{
-    struct os_mbuf_pool *pool;
-    struct os_mbuf_pool *pool_with_free_blocks = NULL;
-    uint16_t pool_free_blocks;
-
-    STAILQ_FOREACH(pool, &g_msys_pool_list, omp_next) {
-        pool_free_blocks = pool->omp_pool->mp_num_free;
-        if (pool_free_blocks != 0) {
-            pool_with_free_blocks = pool;
-            if (dsize <= pool->omp_databuf_len) {
-                break;
-            }
-        }
-    }
-
-    return pool_with_free_blocks;
-}
-
-static struct os_mbuf_pool *
-os_msys_find_biggest_pool(void)
-{
-    return os_msys_find_pool(0xFFFF);
-}
-
-struct os_mbuf *
-os_msys_get(uint16_t dsize, uint16_t leadingspace)
-{
-    struct os_mbuf *m;
-    struct os_mbuf_pool *pool;
-
-    /* If dsize = 0 that means user has no idea how big block size is needed,
-     * therefore lets find for him the biggest one
-     */
-    if (dsize == 0) {
-        pool = os_msys_find_biggest_pool();
-    } else {
-        pool = os_msys_find_pool(dsize);
-    }
-
-    if (!pool) {
-        goto err;
-    }
-
-    m = os_mbuf_get(pool, leadingspace);
-    return (m);
-err:
-    return (NULL);
-}
-
-struct os_mbuf *
-os_msys_get_pkthdr(uint16_t dsize, uint16_t user_hdr_len)
-{
-    uint16_t total_pkthdr_len;
-    struct os_mbuf *m;
-    struct os_mbuf_pool *pool;
-
-    total_pkthdr_len =  user_hdr_len + sizeof(struct os_mbuf_pkthdr);
-
-    /* If dsize = 0 that means user has no idea how big block size is needed,
-     * therefore lets find for him the biggest one
-     */
-    if (dsize == 0) {
-        pool = os_msys_find_biggest_pool();
-    } else {
-        pool = os_msys_find_pool(dsize + total_pkthdr_len);
-    }
-
-    if (!pool) {
-        goto err;
-    }
-
-    m = os_mbuf_get_pkthdr(pool, user_hdr_len);
-    return (m);
-err:
-    return (NULL);
-}
-
-int
-os_msys_count(void)
-{
-    struct os_mbuf_pool *omp;
-    int total;
-
-    total = 0;
-    STAILQ_FOREACH(omp, &g_msys_pool_list, omp_next) {
-        total += omp->omp_pool->mp_num_blocks;
-    }
-
-    return total;
-}
-
-int
-os_msys_num_free(void)
-{
-    struct os_mbuf_pool *omp;
-    int total;
-
-    total = 0;
-    STAILQ_FOREACH(omp, &g_msys_pool_list, omp_next) {
-        total += omp->omp_pool->mp_num_free;
-    }
-
-    return total;
-}
-
-
 int
 os_mbuf_pool_init(struct os_mbuf_pool *omp, struct os_mempool *mp,
                   uint16_t buf_len, uint16_t nbufs)
@@ -275,8 +137,8 @@ os_mbuf_get(struct os_mbuf_pool *omp, uint16_t leadingspace)
 {
     struct os_mbuf *om;
 
-    os_trace_api_u32x2(OS_TRACE_ID_MBUF_GET, (uint32_t)omp,
-                       (uint32_t)(uintptr_t)leadingspace);
+    os_trace_api_u32x2(OS_TRACE_ID_MBUF_GET, (uintptr_t)omp,
+                       (uint32_t)leadingspace);
 
     if (leadingspace > omp->omp_databuf_len) {
         om = NULL;
@@ -296,7 +158,7 @@ os_mbuf_get(struct os_mbuf_pool *omp, uint16_t leadingspace)
     om->om_omp = omp;
 
 done:
-    os_trace_api_ret_u32(OS_TRACE_ID_MBUF_GET, (uint32_t)(uintptr_t)om);
+    os_trace_api_ret_u32(OS_TRACE_ID_MBUF_GET, (uintptr_t)om);
     return om;
 }
 
@@ -307,7 +169,7 @@ os_mbuf_get_pkthdr(struct os_mbuf_pool *omp, uint8_t user_pkthdr_len)
     struct os_mbuf_pkthdr *pkthdr;
     struct os_mbuf *om;
 
-    os_trace_api_u32x2(OS_TRACE_ID_MBUF_GET_PKTHDR, (uint32_t)(uintptr_t)omp,
+    os_trace_api_u32x2(OS_TRACE_ID_MBUF_GET_PKTHDR, (uintptr_t)omp,
                        (uint32_t)user_pkthdr_len);
 
     /* User packet header must fit inside mbuf */
@@ -329,7 +191,7 @@ os_mbuf_get_pkthdr(struct os_mbuf_pool *omp, uint8_t user_pkthdr_len)
     }
 
 done:
-    os_trace_api_ret_u32(OS_TRACE_ID_MBUF_GET_PKTHDR, (uint32_t)(uintptr_t)om);
+    os_trace_api_ret_u32(OS_TRACE_ID_MBUF_GET_PKTHDR, (uintptr_t)om);
     return om;
 }
 
@@ -338,7 +200,7 @@ os_mbuf_free(struct os_mbuf *om)
 {
     int rc;
 
-    os_trace_api_u32(OS_TRACE_ID_MBUF_FREE, (uint32_t)(uintptr_t)om);
+    os_trace_api_u32(OS_TRACE_ID_MBUF_FREE, (uintptr_t)om);
 
     if (om->om_omp != NULL) {
         rc = os_memblock_put(om->om_omp->omp_pool, om);
@@ -360,7 +222,7 @@ os_mbuf_free_chain(struct os_mbuf *om)
     struct os_mbuf *next;
     int rc;
 
-    os_trace_api_u32(OS_TRACE_ID_MBUF_FREE_CHAIN, (uint32_t)(uintptr_t)om);
+    os_trace_api_u32(OS_TRACE_ID_MBUF_FREE_CHAIN, (uintptr_t)om);
 
     while (om != NULL) {
         next = SLIST_NEXT(om, om_next);
@@ -522,12 +384,13 @@ os_mbuf_dup(struct os_mbuf *om)
     struct os_mbuf *head;
     struct os_mbuf *copy;
 
-    omp = om->om_omp;
-
     head = NULL;
     copy = NULL;
 
     for (; om != NULL; om = SLIST_NEXT(om, om_next)) {
+
+        omp = om->om_omp;
+
         if (head) {
             SLIST_NEXT(copy, om_next) = os_mbuf_get(omp,
                     OS_MBUF_LEADINGSPACE(om));
@@ -588,7 +451,7 @@ os_mbuf_off(const struct os_mbuf *om, int off, uint16_t *out_off)
 }
 
 int
-os_mbuf_copydata(const struct os_mbuf *m, int off, int len, void *dst)
+os_mbuf_copydata(const struct os_mbuf *om, int off, int len, void *dst)
 {
     unsigned int count;
     uint8_t *udst;
@@ -600,36 +463,38 @@ os_mbuf_copydata(const struct os_mbuf *m, int off, int len, void *dst)
     udst = dst;
 
     while (off > 0) {
-        if (!m) {
+        if (!om) {
             return (-1);
         }
 
-        if (off < m->om_len)
+        if (off < om->om_len) {
             break;
-        off -= m->om_len;
-        m = SLIST_NEXT(m, om_next);
+        }
+        off -= om->om_len;
+        om = SLIST_NEXT(om, om_next);
     }
-    while (len > 0 && m != NULL) {
-        count = min(m->om_len - off, len);
-        memcpy(udst, m->om_data + off, count);
+    while (len > 0 && om != NULL) {
+        count = min(om->om_len - off, len);
+        memcpy(udst, om->om_data + off, count);
         len -= count;
         udst += count;
         off = 0;
-        m = SLIST_NEXT(m, om_next);
+        om = SLIST_NEXT(om, om_next);
     }
 
     return (len > 0 ? -1 : 0);
 }
 
 void
-os_mbuf_adj(struct os_mbuf *mp, int req_len)
+os_mbuf_adj(struct os_mbuf *om, int req_len)
 {
     int len = req_len;
     struct os_mbuf *m;
     int count;
 
-    if ((m = mp) == NULL)
+    if ((m = om) == NULL) {
         return;
+    }
     if (len >= 0) {
         /*
          * Trim from head.
@@ -645,8 +510,9 @@ os_mbuf_adj(struct os_mbuf *mp, int req_len)
                 len = 0;
             }
         }
-        if (OS_MBUF_IS_PKTHDR(mp))
-            OS_MBUF_PKTHDR(mp)->omp_len -= (req_len - len);
+        if (OS_MBUF_IS_PKTHDR(om)) {
+            OS_MBUF_PKTHDR(om)->omp_len -= (req_len - len);
+        }
     } else {
         /*
          * Trim from tail.  Scan the mbuf chain,
@@ -665,8 +531,9 @@ os_mbuf_adj(struct os_mbuf *mp, int req_len)
         }
         if (m->om_len >= len) {
             m->om_len -= len;
-            if (OS_MBUF_IS_PKTHDR(mp))
-                OS_MBUF_PKTHDR(mp)->omp_len -= len;
+            if (OS_MBUF_IS_PKTHDR(om)) {
+                OS_MBUF_PKTHDR(om)->omp_len -= len;
+            }
             return;
         }
         count -= len;
@@ -677,7 +544,7 @@ os_mbuf_adj(struct os_mbuf *mp, int req_len)
          * Find the mbuf with last data, adjust its length,
          * and toss data from remaining mbufs on chain.
          */
-        m = mp;
+        m = om;
         if (OS_MBUF_IS_PKTHDR(m))
             OS_MBUF_PKTHDR(m)->omp_len = count;
         for (; m; m = SLIST_NEXT(m, om_next)) {
@@ -1273,4 +1140,4 @@ os_mbuf_pack_chains(struct os_mbuf *m1, struct os_mbuf *m2)
     }
 
     return m1;
-}
\ No newline at end of file
+}
