@@ -29,6 +29,10 @@
 
 #undef MYNEWT_VAL_LINK_TEMPLATE
 
+#ifndef MYNEWT_VAL_MAIN_STACK_FILL
+#define MYNEWT_VAL_MAIN_STACK_FILL (0)
+#endif
+
 #ifndef MYNEWT_VAL_MAIN_STACK_SIZE
 #define MYNEWT_VAL_MAIN_STACK_SIZE (768)
 #endif
@@ -577,6 +581,10 @@
 #define MYNEWT_VAL_QSPI_WRITEOC (0)
 #endif
 
+#ifndef MYNEWT_VAL_QSPI_XIP_OFFSET
+#define MYNEWT_VAL_QSPI_XIP_OFFSET (0x10000000)
+#endif
+
 #ifndef MYNEWT_VAL_SPI_0_MASTER
 #define MYNEWT_VAL_SPI_0_MASTER (0)
 #endif
@@ -747,6 +755,84 @@
 #define MYNEWT_VAL_XTAL_RC (0)
 #endif
 
+#ifndef MYNEWT_VAL_JLINK_TARGET
+#define MYNEWT_VAL_JLINK_TARGET (NRF52)
+#endif
+
+#ifndef MYNEWT_VAL_MYNEWT_DEBUGGER__jlink
+#define MYNEWT_VAL_MYNEWT_DEBUGGER__jlink (0)
+#endif
+#ifndef MYNEWT_VAL_MYNEWT_DEBUGGER__openocd
+#define MYNEWT_VAL_MYNEWT_DEBUGGER__openocd (0)
+#endif
+#ifndef MYNEWT_VAL_MYNEWT_DEBUGGER__pyocd
+#define MYNEWT_VAL_MYNEWT_DEBUGGER__pyocd (0)
+#endif
+#ifndef MYNEWT_VAL_MYNEWT_DEBUGGER__st_link_gdbserver
+#define MYNEWT_VAL_MYNEWT_DEBUGGER__st_link_gdbserver (0)
+#endif
+#ifndef MYNEWT_VAL_MYNEWT_DEBUGGER__stutil
+#define MYNEWT_VAL_MYNEWT_DEBUGGER__stutil (0)
+#endif
+#undef MYNEWT_VAL_MYNEWT_DEBUGGER
+
+#undef MYNEWT_VAL_MYNEWT_DEBUGGER_SN
+
+#ifndef MYNEWT_VAL_MYNEWT_DOWNLOADER__bsp_downloader
+#define MYNEWT_VAL_MYNEWT_DOWNLOADER__bsp_downloader (0)
+#endif
+#ifndef MYNEWT_VAL_MYNEWT_DOWNLOADER__ezflashcli
+#define MYNEWT_VAL_MYNEWT_DOWNLOADER__ezflashcli (0)
+#endif
+#ifndef MYNEWT_VAL_MYNEWT_DOWNLOADER__jlink
+#define MYNEWT_VAL_MYNEWT_DOWNLOADER__jlink (0)
+#endif
+#ifndef MYNEWT_VAL_MYNEWT_DOWNLOADER__nrfjprog
+#define MYNEWT_VAL_MYNEWT_DOWNLOADER__nrfjprog (0)
+#endif
+#ifndef MYNEWT_VAL_MYNEWT_DOWNLOADER__nrfutil
+#define MYNEWT_VAL_MYNEWT_DOWNLOADER__nrfutil (1)
+#endif
+#ifndef MYNEWT_VAL_MYNEWT_DOWNLOADER__openocd
+#define MYNEWT_VAL_MYNEWT_DOWNLOADER__openocd (0)
+#endif
+#ifndef MYNEWT_VAL_MYNEWT_DOWNLOADER__pyocd
+#define MYNEWT_VAL_MYNEWT_DOWNLOADER__pyocd (0)
+#endif
+#ifndef MYNEWT_VAL_MYNEWT_DOWNLOADER__stflash
+#define MYNEWT_VAL_MYNEWT_DOWNLOADER__stflash (0)
+#endif
+#ifndef MYNEWT_VAL_MYNEWT_DOWNLOADER__stm32_programmer_cli
+#define MYNEWT_VAL_MYNEWT_DOWNLOADER__stm32_programmer_cli (0)
+#endif
+#ifndef MYNEWT_VAL_MYNEWT_DOWNLOADER
+#define MYNEWT_VAL_MYNEWT_DOWNLOADER (1)
+#endif
+
+#ifndef MYNEWT_VAL_MYNEWT_DOWNLOADER_MFG_IMAGE_FLASH_OFFSET
+#define MYNEWT_VAL_MYNEWT_DOWNLOADER_MFG_IMAGE_FLASH_OFFSET (0x0)
+#endif
+
+#undef MYNEWT_VAL_MYNEWT_DOWNLOADER_OPENOCD_BOARD
+
+#undef MYNEWT_VAL_MYNEWT_DOWNLOADER_OPENOCD_CFG
+
+#undef MYNEWT_VAL_MYNEWT_DOWNLOADER_OPENOCD_INTERFACE
+
+#undef MYNEWT_VAL_MYNEWT_DOWNLOADER_SCRIPT
+
+#undef MYNEWT_VAL_NRFJPROG_COPROCESSOR
+
+#undef MYNEWT_VAL_NRFUTIL_DFU_MODE
+
+#undef MYNEWT_VAL_NRFUTIL_DFU_SN
+
+#undef MYNEWT_VAL_NRFUTIL_TRAITS
+
+#ifndef MYNEWT_VAL_PYOCD_TARGET
+#define MYNEWT_VAL_PYOCD_TARGET (nrf52840)
+#endif
+
 #ifndef MYNEWT_VAL_FLOAT_USER
 #define MYNEWT_VAL_FLOAT_USER (0)
 #endif
@@ -1021,6 +1107,10 @@
 #define MYNEWT_VAL_MODLOG_SYSINIT_STAGE (100)
 #endif
 
+#ifndef MYNEWT_VAL_MODLOG_USE_PRINTF_ATTRIBUTE
+#define MYNEWT_VAL_MODLOG_USE_PRINTF_ATTRIBUTE (0)
+#endif
+
 #ifndef MYNEWT_VAL_LOG_CONSOLE
 #define MYNEWT_VAL_LOG_CONSOLE (1)
 #endif
@@ -1233,6 +1323,10 @@
 #define MYNEWT_VAL_BLE_LL_ADD_STRICT_SCHED_PERIODS (0)
 #endif
 
+#ifndef MYNEWT_VAL_BLE_LL_ADV_CODING_SELECTION
+#define MYNEWT_VAL_BLE_LL_ADV_CODING_SELECTION (0)
+#endif
+
 #ifndef MYNEWT_VAL_BLE_LL_CFG_FEAT_CONN_PARAM_REQ
 #define MYNEWT_VAL_BLE_LL_CFG_FEAT_CONN_PARAM_REQ (MYNEWT_VAL_BLE_LL_ROLE_CENTRAL || MYNEWT_VAL_BLE_LL_ROLE_PERIPHERAL)
 #endif
@@ -1429,6 +1523,10 @@
 #define MYNEWT_VAL_BLE_LL_HCI_VS_LOCAL_IRK (0)
 #endif
 
+#ifndef MYNEWT_VAL_BLE_LL_HCI_VS_SET_SCAN_CFG
+#define MYNEWT_VAL_BLE_LL_HCI_VS_SET_SCAN_CFG (0)
+#endif
+
 #ifndef MYNEWT_VAL_BLE_LL_ISO
 #define MYNEWT_VAL_BLE_LL_ISO (0)
 #endif
@@ -1549,6 +1647,10 @@
 #define MYNEWT_VAL_BLE_LL_SCA (60)
 #endif
 
+#ifndef MYNEWT_VAL_BLE_LL_SCAN_ACTIVE_SCAN_NRPA
+#define MYNEWT_VAL_BLE_LL_SCAN_ACTIVE_SCAN_NRPA (0)
+#endif
+
 #ifndef MYNEWT_VAL_BLE_LL_SCAN_AUX_SEGMENT_CNT
 #define MYNEWT_VAL_BLE_LL_SCAN_AUX_SEGMENT_CNT (0)
 #endif
@@ -2405,6 +2507,7 @@
 #define MYNEWT_PKG_apache_mynewt_core__hw_mcu_nordic 1
 #define MYNEWT_PKG_apache_mynewt_core__hw_mcu_nordic_nrf52xxx 1
 #define MYNEWT_PKG_apache_mynewt_core__hw_mcu_nordic_nrf_common 1
+#define MYNEWT_PKG_apache_mynewt_core__hw_scripts 1
 #define MYNEWT_PKG_apache_mynewt_core__kernel_os 1
 #define MYNEWT_PKG_apache_mynewt_core__libc 1
 #define MYNEWT_PKG_apache_mynewt_core__libc_baselibc 1
